// -*- Mode: c++ -*-
//Header:
//File: SString.h
//Author: NODA, Itsuki
//Date: 1999/05/18
//

//ModifyHistory:
// 1999/05/18: Start to create this file
// 1999/11/07: introduce Itk namespace
// 1999/11/07: Divide each class to separate file
//EndModifyHistory:

/*
 * Copyright (C) 2001 NODA, Itsuki, CARC, AIST, JAPAN
 * Copyright (C) 1999, 2000 Itsuki Noda, Electrotechnical Laboratory, Japan
 */

#ifndef _itk_SString_h_
#define _itk_SString_h_
//////////////////////////////////////////////////////////////////////
#include "itk/btype.h"
#include "itk/utility.h"
#include <stdio.h>
#include <stdarg.h>


namespace Itk {

   //======================================================================
   // SString Class
   //

   class SubString {
      public:
	 char* head ;
	 char* tail ;

	 //--------------------------------------------------
	 // static member

	 //static const SubString null() { return SubString() ; } ;
	 //static const SubString zero() { return SubString("") } ;

	 //--------------------------------------------------
	 // constructor

	 SubString(const UInt size) { alloc(size) ; } ;
	 SubString(char* h, char* t) { set(h,t) ; } ;
	 SubString(char* h, const UInt l) { set(h,l) ; } ;
	 SubString(char* str = ITK_NULLPTR) { set(str) ; } ;
	 SubString(const SubString& src) { set(src) ; } ;
	 SubString(const SubString& src, 
		   const UInt hidx, const UInt tidx) {
	    set(src,hidx,tidx) ;
	 } ;

	 //--------------------------------------------------
	 // operator=

	 SubString& operator= (const SubString& s) { 
	    set(s) ; return *this ;
	 } ;

	 SubString& operator= (char* s) {
	    set(s) ; return *this ;
	 } ;

	 SubString& operator= (const char * s) {
	    set(s) ; return *this ;
	 } ;

	 //--------------------------------------------------
	 // operator==

	 Bool eq(const SubString& r) const ;

	 Bool equal(const SubString& r) const ;

	 Bool operator== (const SubString& r) const ;

	 Bool operator== (const char* ref) const ;

	 //--------------------------------------------------
	 // prefix check

	 Bool isPrefixOf(const SubString& ref) const ;

	 Bool isPrefixOf(const char* const ref) const ;

	 Bool hasPrefix(const SubString& prefix) const ;

	 Bool hasPrefix(const char* const ref) const ;

	 Bool checkPopPrefix(const SubString & prefix) ;

	 Bool checkPopPrefix(const char * const prefix) ;

	 //--------------------------------------------------
	 // index (return -1 if not exist)

	 Int index(const char c) const ;

	 Bool has(const char c) const ;

	 //--------------------------------------------------
	 // set

	 void set(char* h, char* t) { head = h ;  tail = t ; } ;
	 void set(char* h, const UInt l) { 
	    head = h ; setLength(l) ;
	 } ;
	 void set(char* str = ITK_NULLPTR) { 
	    if(isNull(str)) set((char*)ITK_NULLPTR,(char*)ITK_NULLPTR) ;
	    else set(str,strlen(str)) ;
	 } ;
	 void set(const char * str) { set(::strdup(str)) ; } ;

	 void set(const SubString& src) { set(src.head,src.tail) ; } ;
	 void set(const SubString& src, const UInt hidx, const UInt tidx) {
	    head = &(src.head[hidx]) ;
	    tail = &(src.head[tidx]) ;
	 } ;
	 void set(const SubString& src, const UInt hidx) {
	    head = &(src.head[hidx]) ;
	    tail = src.tail ;
	 } ;

	 //--------------------------------------------------
	 // setNull & check Null
	 void setNull() { set((char*)ITK_NULLPTR, (char*)ITK_NULLPTR) ; } ;
	 Bool isNullPtr() const { return isNull(head) ; } ;

	 //--------------------------------------------------
	 // alloc
	 Bool alloc(const UInt l) ;

	 //--------------------------------------------------
	 // length
	 UInt length() const { return (tail - head) ; } ;
	 UInt size() const { return length() + 1 ; } ;

	 void setLength(const UInt l) ;
      
	 UInt strlength() const ;

	 void adjustLength() ;

	 //--------------------------------------------------
	 // copy

	 Bool copy(const SubString& src, Bool allocp = False) ;
	 Bool copy(char* h, char* t, Bool allocp = False) ;
	 Bool copy(const char * h, const char * t = ITK_NULLPTR) ;

	 //--------------------------------------------------
	 // dup

	 SubString dup() const ;

	 char * strdup() const ;
      
	 //--------------------------------------------------
	 // copy to char*

	 void copyTo(char* dst) const ;

	 //--------------------------------------------------
	 // converter

	 // operator const char* () const { return head ; } ;
	 operator char* () const { return head ; } ;
      
	 //--------------------------------------------------
	 // terminate
	 void terminate() { *tail = ITK_NULLCHR ; } ;

	 Bool terminatedp() const { return isTerminated() ; } ;
	 Bool isTerminated() const { return strlength() <= length() ; } ; 

	 Bool isNullStr() const { return head == tail ; } ;
      
	 //--------------------------------------------------
	 // basic operation (get)

	 char& first() const { return *head ; } ;
	 char& last() const { return *(tail - 1) ; } ;

	 Bool isFirst(const char c) const { return first() == c ; } ;
      
	 Bool tailp() const { return head >= tail ; } ;
	 Bool nullp() const { return head >= tail ; } ;

	 char popChar() ;
	 char getChar() { return popChar() ; } ; 

	 void chop() { tail-- ; } ;
	 void ungetChar() { head-- ; } ;

	 void ignoreChar() { head++ ; } ;
	 Bool skipNChar(UInt n = 1) ;

	 UInt skipWhile(const SubString & charlist) ;
	 UInt skipUntil(const SubString & charlist) ;

	 SubString sub(UInt length) const { return sub(0, length) ; }
	 SubString sub(UInt offset, UInt length) const ;

	 SubString popSubString(UInt length) { 
	    return popSubString(0,length) ; 
	 } ;
	 SubString popSubString(UInt offset, UInt length) ;

	 SubString shiftedSubString(UInt offset) const {
	    return sub(offset,length()-offset) ;
	 } ;
      
	 //--------------------------------------------------
	 // basic operation (put)

	 void pushChar(const char c) ;

	 void putChar(const char c) ;

	 void put(const SubString& str) ;

	 void put(const char* str) { 
	    put(SubString(const_cast<char*>(str))) ; } ;

	 void put(const char* str, UInt len) { 
	    put(SubString(const_cast<char*>(str),len)) ; } ;

	 //--------------------------------------------------
	 // printf1

	 Int printf(const char * format, ...) ;

	 Int printf1(const char* format) ;

	 Int printf1(const char* format, const char * x) ;

	 Int printf1(const char* format, Int x) ;

	 Int printf1(const char* format, UInt x) ;

	 Int printf1(const char* format, Flt x) ;
	 
	 //--------------------------------------------------
	 // concatination

	 SubString append(const SubString & rest) const ;

	 SubString operator+ (const SubString & rest) const {
	    return append(rest) ;
	 } ;

	 SubString operator+ (const char* rest) const {
	    return append(SubString((char*)rest)) ;
	 } ;

	 //--------------------------------------------------
	 // scan by char set

	 SubString scanInCharSet(const SubString & charSet) ;

	 SubString scanInCharSet(char * const charSet) {
	    return scanInCharSet(SubString((char*)charSet)) ;
	 } ;

	 Bool skipInCharSet(const SubString & charSet) ;
	    
	 Bool skipInCharSet(char * const charSet) {
	    return skipInCharSet(SubString((char*)charSet)) ;
	 } ;

	 //--------------------------------------------------
	 // scan by delimiter

	 SubString scanByDelimiter(const SubString & delimiter) ;

	 SubString scanByDelimiter(char * const delimiter) {
	    return scanByDelimiter(SubString((char*)delimiter)) ;
	 } ;

	 //--------------------------------------------------
	 // scan as number value

	 Flt scanAsFlt() ;

	 Int scanAsInt() ;

	 UInt scanAsUInt() ;

	 //--------------------------------------------------
	 // conversion

	 Bool popIntVal(Int& i) ;

	 Bool popUIntVal(UInt& i) ;

	 Bool popFloatVal(Flt& v) ;

	 //--------------------------------------------------
	 // describe

	 void describe(ostream& o, Bool detailp = True) const {
	    if(detailp) {
	       o << "#SubString[<" << (void*)head << ">-<" << (void*)tail
		 << ">(len=" << length() << "):" ;
	    } 
	    if(isNullPtr()) {
	       o << "(null)" ;
	    } else {
	       for(char* s = head ; s < tail ; s++) {
		  o << *s ;
	       }
	    }
	    if(detailp) {
	       o << "]" << endl ;
	    }
	 } ;

	 friend ostream& operator<< (ostream& o, const SubString& s) {
	    s.describe(o,False) ;
	    return o ;
	 } ;
   } ;

   //======================================================================
   // SString Class
   //

   typedef SubString SString ;

} ;


//////////////////////////////////////////////////////////////////////
#endif
