// -*- Mode:C++ -*-
//Header:
//File: FrameSexp.cc
//Author: NODA, Itsuki
//Date: 2001/06/29
//

//ModifyHistory:
// 2001/06/29: Start to create this file
//EndModifyHistory:

#include "itk/SimpleSexp2.h"

namespace Itk {

   //======================================================================
   // class FrameSexp

#define ITK_FRAME_SLOTNAME_PREFIX	":"

   class FrameSexp : public SimpleSexp2 {

	 //------------------------------------------------------------
	 // slotname prefix string (static const)
      public:
	 static const char slotnamePrefix[] = ITK_FRAME_SLOTNAME_PREFIX ;

	 //------------------------------------------------------------
	 // Special Keyword Symbols (static const) 
	 // defines 
	 //	KeySym_xxxx
	 // usage :
	 //   to define KeySum_Foo as symbol ":foo",
	 //     place ITK_FRAME_DEF_KEYSYM_HEAD(Foo,":foo") in class def.
	 //     place ITK_FRAME_DEF_KEYSYM_BODY(Foo) outside of class def.

#define ITK_FRAME_DEF_KEYSYM_HEAD(name,str) \
	public: static const char _KeySymName_##name##[] = str ; \
	public: static SimpleSexp2  _KeySym_##name ; \
	public: static SimpleSexp2 * KeySym_##name ;

#define ITK_FRAME_DEF_KEYSYM_BODY(name) \
	SimpleSexp2 FrameSexp::_KeySym_##name = \
	   SimpleSexp2(SimpleSexp2::T_Atom,\
		       FrameSexp::_KeySymName_##name) ;\
	SimpleSexp2 * FrameSexp::KeySym_##name = &FrameSexp::_KeySym_##name ;

	 ITK_FRAME_DEF_KEYSYM_HEAD(Set,":set") ;
	 ITK_FRAME_DEF_KEYSYM_HEAD(List,":list") ;
	 ITK_FRAME_DEF_KEYSYM_HEAD(Not,":not") ;
	 ITK_FRAME_DEF_KEYSYM_HEAD(Wh,":wh") ;

	 //------------------------------------------------------------
	 // special type frame check
      public:
	 Bool isSetFrame() const {
	    return isCons() && car()->equal(KeySym_Set) ;
	 } ;
	 Bool isListFrame() const {
	    return isCons() && car()->equal(KeySym_List) ;
	 } ;
	 Bool isNotFrame() const {
	    return isCons() && car()->equal(KeySym_Not) ;
	 } ;
	 Bool isWhFrame() const {
	    return isCons() && car()->equal(KeySym_Wh) ;
	 } ;

	 //------------------------------------------------------------
	 // scan wrapper
      public:
	 static FrameSexp * scan(Scanner * scanner, Bool resetp = True) {
	    return 
	       static_cast<FrameSexp *>(SimpleSexp2::scan(scanner,resetp)) ;
	 } ;
      public:
	 static FrameSexp * scan(Scanner & scanner, Bool resetp = True) {
	    return 
	       static_cast<FrameSexp *>(SimpleSexp2::scan(scanner,resetp)) ;
	 } ;
      public:
	 static FrameSexp * scan(istream * istr, Bool resetp = True) {
	    return 
	       static_cast<FrameSexp *>(SimpleSexp2::scan(istr,resetp)) ;
	 } ;
      public:
	 static FrameSexp * scan(istream & istr, Bool resetp = True) {
	    return 
	       static_cast<FrameSexp *>(SimpleSexp2::scan(istr,resetp)) ;
	 } ;
      public:
	 static FrameSexp * scan(Buffer & buf, Bool resetp = True) {
	    return 
	       static_cast<FrameSexp *>(SimpleSexp2::scan(buf,resetp)) ;
	 } ;
      public:
	 static FrameSexp * scan(Buffer * buf, Bool resetp = True) {
	    return 
	       static_cast<FrameSexp *>(SimpleSexp2::scan(buf,resetp)) ;
	 } ;
      public:
	 static FrameSexp * scan(const char * str, Bool resetp = True) {
	    return 
	       static_cast<FrameSexp *>(SimpleSexp2::scan(str,resetp)) ;
	 } ;
	 
	 //------------------------------------------------------------
	 // syntax check
      public:
	 Bool chkSyntax(Int depth = -1) const ;

	 //------------------------------------------------------------
	 // get slot value
      public:
	 FrameSexp * slotvalue(const SubString & slotname) {
	    if(!isCons()) return static_cast<FrameSexp*>(Nil) ;
	    return static_cast<FrameSexp*>(cdr()->passoc(slotname)) ;
	 }
	 FrameSexp * slotvalue(const SimpleSexp2 * slotname) {
	    if(!isCons()) return static_cast<FrameSexp*>(Nil) ;
	    return static_cast<FrameSexp*>(cdr()->passoc(slotname)) ;
	 }
   } ;

   //======================================================================
   // class FrameSexp's special symbols (class-variables)

   ITK_FRAME_DEF_KEYSYM_BODY(Set) ;
   ITK_FRAME_DEF_KEYSYM_BODY(List) ;
   ITK_FRAME_DEF_KEYSYM_BODY(Not) ;
   ITK_FRAME_DEF_KEYSYM_BODY(Wh) ;

   //======================================================================
   // class FrameSexp's method

   //------------------------------------------------------------
   // syntax check
   //      if depth > 0, then check upper than the depth.
   //      if depth < 0, it checks whole frames.
   //      but if depth < 0 and too deep fram (more than 2^32), 
   //      then it stops.

   Bool FrameSexp::chkSyntax(Int depth) const {
      if     (isAtom()) return True ;	// atom is frame
      else if(isNil()) return True ;	// nil is frame
      else if(isCons()) {
	 if(depth == 0) return True ;	// not check cons at bottom

	 SimpleSexp2 * head = car() ;
	 SimpleSexp2 * args = cdr() ;
	 if      (isNotFrame() || 
		  isWhFrame()) { 	// for one arg keyword
	    if(args->length() != 1) {
	       ITK_WRN(ITK_HERE <<
		       "keyword " << head << " should have just one arg." << 
		       " [" << this << "]") ;
	       return False ;
	    }
	    return static_cast<FrameSexp*>(args->car())->chkSyntax(depth - 1) ;
	 } 
	 else if (isSetFrame() ||
		  isListFrame()) {	// for multi arg keyword
	    for(args = args ;
		args->isNil() ;
		args = args->cdr()) {
	       if(!args->isCons()) {
		  ITK_WRN(ITK_HERE <<
			  "A frame must not be terminated by non-nil." <<
			  " [" << this << "]") ;
		  return False ;
	       }
	       if(!static_cast<FrameSexp*>(args->car())->chkSyntax(depth - 1))
		  return False ;		// inner error
	    } 
	 }
	 else { 				// general case
	    for(args = args ;
		args->isNil() ;
		args = args->cdr()) {

	       if(!args->isCons()) {
		  ITK_WRN(ITK_HERE <<
			  "A frame must not be terminated by non-nil." <<
			  " [" << this << "]") ;
		  return False ;
	       }

	       SimpleSexp2 * slotname = args->car() ;
	       if(!slotname->isAtom()) {
		  ITK_WRN(ITK_HERE <<
			  "A slotname should be an atom. :" << slotname <<
			  " [" << this << "]") ;
		  return False ;
	       }
	       if(!slotname->scanAsConstSymbol().hasPrefix(slotnamePrefix)) {
		  ITK_WRN(ITK_HERE <<
			  "A slotname should start by '" << slotnamePrefix <<
			  "'. : " << slotname <<
			  " [" << this << "]") ;
		  return False ;
	       }
	    
	       args = args->cdr() ;

	       if(!args->isCons()) {
		  ITK_WRN(ITK_HERE <<
			  "A frame must have slot-value pairs." <<
			  " [" << this << "]") ;
		  return False ;
	       }

	       FrameSexp * slotvalue = static_cast<FrameSexp*>(args->car()) ;
	       if(!slotvalue->chkSyntax(depth - 1)) {
		  return False ;
	       }
	    }
	 }
	 return True ;  // when all args is fine, this is a frame.
      }
      else {
	 ITK_WRN(ITK_HERE << "unknown syntax error." << 
		 " [" << this << "]") ;
	 return False ;
      }
   }
   
}

//======================================================================
// for test
#if 1

using namespace Itk ;

int main() {
   FrameSexp * fs =
      FrameSexp::scan("(foo :bar (foo1 :bar () :baz 7) :boo 3
		        :a (:list a b c d e))") ;
   ITK_DBG(fs) ;

   ITK_DBG(fs->chkSyntax()) ;
   
   ITK_DBG(fs->slotvalue(":bar")) ;
   ITK_DBG(fs->slotvalue(SString(":bar"))->slotvalue(":baz")) ;


} ;

#endif
