// -*- Mode:C++ -*-
//Header:
//File: BpLLink.h
//Author: NODA, Itsuki
//Date: 1999/12/16
//

//ModifyHistory:
// 1999/12/16: Start to create this file
// 1999/12/26: reconfigulation
//EndModifyHistory:

#ifndef _Nnn_BpLLink_h_
#define _Nnn_BpLLink_h_
////////////////////////////////////////////////////////////////////////

#include "n3btype.h"
#include "n3utility.h"
#include "StrForm.h"
#include "BpUnitBase.h"
#include "BpLinkBase.h"
#include "BpLayer.h"

namespace Nnn {

   using namespace Itk ;

   //======================================================================
   // BpLLink (Layer-Link)

   class BpLLinkTable ;

   class BpLLink : public WithDescriber {

	 //--------------------------------------------------
	 // label string for save/load/scan
      public:
	 static char* const sformTermHead = "BpLLink" ; 
	 static char* const sformFromLbl = "from" ; 
	 static char* const sformToLbl = "to" ; 
	 static char* const sformToSizeLbl = "n" ; 
	 static char* const sformFromSizeLbl = "m" ; 
	 static char* const sformLinkLbl = "link" ; 
	 static char* const sformInitLbl = "init" ; 
	 static char* const sformLrnRateLbl = "lrate" ; 
	 static char* const sformEquivLbl = "equiv" ;

	 //--------------------------------------------------
	 // member
      public:
	 SubString fromname ;
	 SubString toname ;
	 BpLayer * from ;
	 BpLayer * to ;
	 Size n ;
	 Size m ;
	 BpLinkBase *_link ;
	 StrForm * initInfo ;
	 BpLLink * equiv ;
	 StrForm * equivInfo ;
	 Val lrate ; // rate of learning. multiplied with eps

	 //--------------------------------------------------
	 // constructor
      public:
	 BpLLink() { init() ; } ;
	 BpLLink(BpLayer * fromLayer, BpLayer * toLayer) {
	    init(fromLayer,toLayer) ;
	 } ;

	 //--------------------------------------------------
	 // initialize
      public:
	 void init(BpLayer * fromLayer = ITK_NULLPTR,
		   BpLayer * toLayer = ITK_NULLPTR,
		   BpLinkBase * l = ITK_NULLPTR) {
	    n = 0 ; m = 0 ;
	    from = fromLayer ;
	    to = toLayer ;
	    if(!isNull(from)) {
	       fromname = from->name ;
	       m = from->n ;
	    } 
	    if(!isNull(to)) {
	       toname = to->name ;
	       n = to->n ;
	    }
	    if(isNull(l)) {
	       if(n * m > 0) {
		  _link = new BpLinkBase[n*m] ;
	       } else {
		  _link = ITK_NULLPTR ;
	       } 
	    } else {
	       _link = l ;
	    }
	    lrate = 1.0 ;
	    initInfo = ITK_NULLPTR ;
	    equiv = ITK_NULLPTR ;
	    equivInfo = ITK_NULLPTR ;
	 } ;

	 //--------------------------------------------------
	 // resize
      public:
	 Bool resize(Size newN, Size newM) {
	    Size oldN = n ;
	    Size oldM = m ;
	    n = newN ; m = newM ;
	    if(oldN * oldM > newN * newM) return False ;
	    
	    if(!isNull(_link)) delete _link ;
	    _link = new BpLinkBase[n * m] ;
	    return True ;
	 } ;

	 //--------------------------------------------------
	 // actual index 
      public:
	 Index _idx(const Index & i) const { return i * m ; } ;
      public:
	 Index _idx(const Index & i, const Index & j) const {
	    return (_idx(i) + j) ;
	 } ;

	 //--------------------------------------------------
	 // access
      public:
	 const BpLinkBase & link(const Index & i, const Index & j) const {
	    return _link[_idx(i,j)] ;
	 } ;
	 BpLinkBase & link(const Index & i, const Index & j) { 
	    return _link[_idx(i,j)] ; 
	 } ;

	 const BpLinkBase * link(const Index & i) const {
	    return &(_link[_idx(i)]) ;
	 } ;

	 BpLinkBase * link(const Index & i) { return &(_link[_idx(i)]) ; } ;

	 //--------------------------------------------------
	 // processing
      public:
	 void propFore() ;
	 void propBack() ;

	 //--------------------------------------------------
	 // learning
      public:
	 Val learn(const Val & eps, const Val & alpha) ;

	 //--------------------------------------------------
	 // randomize weight 
      public:
	 void randomizeWeight(const Val & minval, const Val & maxval) ;

	 //--------------------------------------------------
	 // save 
      public:
	 virtual void save(ostream & ostr,
			   const Bool & newlinep = True,
			   const SubString & indent = SubString("")) const ;

	 //--------------------------------------------------
	 // load
      public:
	 virtual void load(istream & istr, 
			   BpLayerTable * layerTable = ITK_NULLPTR,
			   BpLLinkTable * linkTable = ITK_NULLPTR) {
	    StrForm * sform = StrForm::scan(istr) ;
	    scan(*sform, layerTable, linkTable) ;
	    delete sform ;
	 } ;

	 //--------------------------------------------------
	 // scan
      public:
	 virtual void scan(StrForm & sform,
			   BpLayerTable * layerTable = ITK_NULLPTR,
			   BpLLinkTable * linkTable = ITK_NULLPTR) ;

	 //--------------------------------------------------
	 // fix from&to layer
      public:
	 Bool fixFromTo(const BpLayerTable & layerTable) ;

	 //--------------------------------------------------
	 // registerFromTo (register link information into From/To layer)
      public:
	 void registerFromTo() ;

	 //--------------------------------------------------
	 // fix Equiv Link
      public:
	 Bool fixEquiv(const BpLLinkTable & layerTable) ;

	 //--------------------------------------------------
	 // set equiv
      public:
	 void setEquiv(BpLLink * lnk) ;

	 //--------------------------------------------------
	 // describe
      public:
	 virtual void describe(ostream & ostr, 
			       const Bool detailp = True) const {
	    ostr << "#BpLLink[" 
		 << (isNull(from) ? "(null)" : (const char *)from->name)
		 << "->"
		 << (isNull(to) ? "(null)" : (const char *)to->name)
		 << "]" ;

	    if(detailp) {
	       ostr << endl ;

	       ostr << "  " << "from=" ;
	       if(isNull(from)) ostr << "(null)" ;
	       else             ostr << *from ;
	       ostr << endl ;

	       ostr << "  " << "to=" ;
	       if(isNull(from)) ostr << "(null)" ;
	       else             ostr << *to ;
	       ostr << endl ;

	       if(!isNull(equiv)) {
		  ostr << "  " << "equiv=" << *equiv << endl ;
	       }

	       ostr << "  " << "size=(" << n << "," << m << ")" << endl ;
	       ostr << "  " << "w=(" ;
	       if(isNull(_link)) ostr << "null" ;
	       else {
		  for(Index i = 0 ; i < n ; i++) {
		     if(i > 0) ostr << endl << "     ";
		     for(Index j = 0 ; j < m ; j++) {
			if(j > 0) ostr << ", " ;
			ostr << NNN_VAL_FORMAT << link(i,j).w ;
		     }
		  }
	       }
	       ostr << ")" << endl ;
	    } 
	 } ;
   } ;

   //======================================================================
   // BpLLink Table

   //--------------------------------------------------
   // Index

   class BpLLinkIndex : public WithDescriber {
	 //--------------------------------------------------
	 // member
      public:
	 SString from ;
	 SString to ;
	 
	 //--------------------------------------------------
	 // constructor
      public:
	 BpLLinkIndex() : from(), to() {} ;
	 BpLLinkIndex(const SString & f, const SString & t) {
	    from = f ; to = t ;
	 } ;
	 BpLLinkIndex(const BpLayer & f, const BpLayer & t) {
	    from = f.name ; to = t.name ;
	 } ;
	 BpLLinkIndex(const BpLayer * f, const BpLayer * t) {
	    from = f->name ; to = t->name ;
	 } ;
	 BpLLinkIndex(const BpLLink * lnk) {
	    from = lnk->from->name ;
	    to = lnk->to->name ;
	 } ;
	 BpLLinkIndex(const BpLLink & lnk) {
	    from = lnk.from->name ;
	    to = lnk.to->name ;
	 } ;
	 
	 //--------------------------------------------------
	 // describe
      public:
	 virtual void describe(ostream & ostr,
			       const Bool detailp = True) const {
	    ostr << "#BpLLlinkIndex[" << from << "->" << to << "]" ;
	    if(detailp) {
	       ostr << endl ;
	    } 
	 } ;
   } ;
   
   inline Bool MatchIndex(const BpLLinkIndex & x, const BpLLinkIndex & y) {
      return x.from == y.from && x.to == y.to ;
   } ;

   //--------------------------------------------------
   // Table

   class BpLLinkTable : public IndexTableT<BpLLinkIndex, BpLLink *> {
	 //--------------------------------------------------
	 // assign
      public:
	 Entry * assign(BpLLink * lnk) {
	    return IndexTable::assign(BpLLinkIndex(lnk),lnk) ;
	 } ;
	 Entry * assign(BpLLink & lnk) { return assign(&lnk) ; } ;

	 //--------------------------------------------------
	 // find
      public:
	 BpLLink * find(const BpLayer * from, const BpLayer * to) const {
	    return IndexTable::find(BpLLinkIndex(from,to)) ;
	 } ;
	 BpLLink * find(const BpLayer & from, const BpLayer & to) const {
	    return find(&from,&to) ;
	 } ;
	 BpLLink * find(const SubString & from, const SubString & to) const {
	    return IndexTable::find(BpLLinkIndex(from,to)) ;
	 } ;
   } ;

} ;

////////////////////////////////////////////////////////////////////////
#endif


