// -*- Mode:C++ -*-
//Header:
//File: BpNet3SigLyr2.h
//Title: BpNet with 3 sigmoid layers (for test)
//Author: NODA, Itsuki
//Date: 2000/01/03
//

//ModifyHistory:
// 2000/01/03: Start to create this file
//EndModifyHistory:

#ifndef _Nnn_BpNet3SigLyr2_h_
#define _Nnn_BpNet3SigLyr2_h_
////////////////////////////////////////////////////////////////////////

#include "BpNetBase.h"

namespace Nnn {

   using namespace Itk ;

   //======================================================================
   // BpNet3SigLyr2

   class BpNet3SigLyr2 : public BpNetBase {

      public:
	 //--------------------------------------------------
	 // constants

	 static char* const _sformTermHead = "BpNet3SigLyr2" ; 
	 virtual char* sformTermHead() const {
	    return _sformTermHead ;
	 } ;

	 static char * const nameInput1 = "input1" ;
	 static char * const nameInput2 = "input2" ;
	 static char * const nameHidden1 = "hidden1" ;
	 static char * const nameHidden2 = "hidden2" ;
	 static char * const nameOutput1 = "output1" ;
	 static char * const nameOutput2 = "output2" ;

	 //--------------------------------------------------
	 // member
      public:
	 BpLayer * lyrInput1 ;
	 BpLayer * lyrInput2 ;
	 BpLayer * lyrHidden1 ;
	 BpLayer * lyrHidden2 ;
	 BpLayer * lyrOutput1 ;
	 BpLayer * lyrOutput2 ;

	 BpLLink * lnkInp1Hid1 ;
	 BpLLink * lnkInp1Hid2 ;
	 BpLLink * lnkInp2Hid1 ;
	 BpLLink * lnkInp2Hid2 ;
	 BpLLink * lnkHid1Out1 ;
	 BpLLink * lnkHid1Out2 ;
	 BpLLink * lnkHid2Out1 ;
	 BpLLink * lnkHid2Out2 ;

	 //--------------------------------------------------
	 // constructor
      public:
	 BpNet3SigLyr2() { init() ; } ;
	 BpNet3SigLyr2(const SString & nm,
		      const UInt & isize,
		      const UInt & hsize,
		      const UInt & osize,
		      const Val & tempval = NNN_DFLT_TEMP,
		      const Val & epsval = NNN_DFLT_EPS,
		      const Val & alphaval = NNN_DFLT_ALPHA) { 
	    init(nm,tempval,epsval,alphaval) ;
	    assignLayerLLink(isize,hsize,osize) ;
	 } ;

	 //--------------------------------------------------
	 // assignLayer
      public:
	 void assignLayerLLink(const UInt & isize, 
			       const UInt & hsize, 
			       const UInt & osize) {
	    lyrInput1 = newLayer(nameInput1,isize,BpLayer::FT_linear) ;
	    lyrInput2 = newLayer(nameInput2,isize,BpLayer::FT_linear) ;
	    lyrHidden1 = newLayer(nameHidden1,hsize,BpLayer::FT_sigmoid) ;
	    lyrHidden2 = newLayer(nameHidden2,hsize,BpLayer::FT_sigmoid) ;
	    lyrOutput1 = newLayer(nameOutput1,osize,BpLayer::FT_sigmoid) ;
	    lyrOutput2 = newLayer(nameOutput2,osize,BpLayer::FT_sigmoid) ;
	    
	    lnkInp1Hid1 = newLLink(lyrInput1,lyrHidden1) ;
	    lnkInp1Hid2 = newLLink(lyrInput1,lyrHidden2) ;
	    lnkInp2Hid1 = newLLink(lyrInput2,lyrHidden1) ;
	    lnkInp2Hid2 = newLLink(lyrInput2,lyrHidden2) ;
	    lnkHid1Out1 = newLLink(lyrHidden1,lyrOutput1) ;
	    lnkHid1Out2 = newLLink(lyrHidden1,lyrOutput2) ;
	    lnkHid2Out1 = newLLink(lyrHidden2,lyrOutput1) ;
	    lnkHid2Out2 = newLLink(lyrHidden2,lyrOutput2) ;
	 } ;

	 //--------------------------------------------------
	 // propFore
      public:
	 void propFore(const ValVector & inpvec) {
	    lyrInput1->propFore(inpvec,temp) ;
	    lyrInput2->propFore(inpvec,temp) ;
	    lyrHidden1->propFore(temp) ;
	    lyrHidden2->propFore(temp) ;
	    lyrOutput1->propFore(temp) ;
	    lyrOutput2->propFore(temp) ;
	 } ;
	    
	 //--------------------------------------------------
	 // propBack
      public:
	 void propBack(const ValVector & tchvec) {
	    lyrOutput1->propBack(tchvec,temp) ;
	    lyrOutput2->propBack(tchvec,temp) ;
	    lyrHidden1->propBack(temp) ;
	    lyrHidden2->propBack(temp) ;
	    // lyrInput->propBack(inpvec,temp) ;
	 } ;

	 //--------------------------------------------------
	 // learn
      public:
	 Flt learn() {
	    Val r = 0.0 ;
	    r += lyrHidden1->learn(eps,alpha) ;
	    r += lyrHidden2->learn(eps,alpha) ;
	    r += lyrOutput1->learn(eps,alpha) ;
	    r += lyrOutput2->learn(eps,alpha) ;
	    r += lnkInp1Hid1->learn(eps,alpha) ;
	    r += lnkInp1Hid2->learn(eps,alpha) ;
	    r += lnkInp2Hid1->learn(eps,alpha) ;
	    r += lnkInp2Hid2->learn(eps,alpha) ;
	    r += lnkHid1Out1->learn(eps,alpha) ;
	    r += lnkHid1Out2->learn(eps,alpha) ;
	    r += lnkHid2Out1->learn(eps,alpha) ;
	    r += lnkHid2Out2->learn(eps,alpha) ;
	    return r ;
	 } ;

	 //--------------------------------------------------
	 // get sum of square error
      public:
	 Flt sumError() const { return lyrOutput1->sumError() ; } ;

	 //--------------------------------------------------
	 // get output
      public:
	 void getNetOutput(ValVector & outvec) const { 
	    lyrOutput1->getNetOutput(outvec) ;
	 } ;

	 //--------------------------------------------------
	 // scan
      public:
	 virtual void scan(StrForm & sform) {
	    BpNetBase::scan(sform) ;
	    lyrInput1 = layer(nameInput1) ;
	    lyrInput2 = layer(nameInput2) ;
	    lyrHidden1 = layer(nameHidden1) ;
	    lyrHidden2 = layer(nameHidden2) ;
	    lyrOutput1 = layer(nameOutput1) ;
	    lyrOutput2 = layer(nameOutput2) ;
	    lnkInp1Hid1 = link(lyrInput1,lyrHidden1) ;
	    lnkInp1Hid2 = link(lyrInput1,lyrHidden2) ;
	    lnkInp2Hid1 = link(lyrInput2,lyrHidden1) ;
	    lnkInp2Hid2 = link(lyrInput2,lyrHidden2) ;
	    lnkHid1Out1 = link(lyrHidden1,lyrOutput1) ;
	    lnkHid1Out2 = link(lyrHidden1,lyrOutput2) ;
	    lnkHid2Out1 = link(lyrHidden2,lyrOutput1) ;
	    lnkHid2Out2 = link(lyrHidden2,lyrOutput2) ;
	 } ;
   } ;

} ;
////////////////////////////////////////////////////////////////////////
#endif


