// -*- Mode:C++ -*-
//Header:
//File: BpNetBase.cc
//Author: NODA, Itsuki
//Date: 1999/12/31
//

//ModifyHistory:
// 1999/12/31: Start to create this file
//EndModifyHistory:

#include "BpNetBase.h"

namespace Nnn {

   //======================================================================
   // allocation

   //----------------------------------------------------------------------
   // new layer

   //--------------------------------------------------
   // by StrForm

   BpLayer * BpNetBase::newLayer(StrForm & sform) {
      BpLayer * layer = new BpLayer ;
      layer->scan(sform) ;
      addLayer(layer) ;
      return layer ;
   } ;

   //--------------------------------------------------
   // by Name

   BpLayer * BpNetBase::newLayer(const SString & nm, UInt size,
				 BpLayer::FuncType ftype) {
      BpLayer * layer = new BpLayer(nm,size,ftype) ;
      addLayer(layer) ;
      return layer ;
   } ;
      

   //----------------------------------------------------------------------
   // add layer

   void BpNetBase::addLayer(BpLayer * layer) {
      layerTable.assign(layer) ;
   } ;

   //----------------------------------------------------------------------
   // get layer

   BpLayer * BpNetBase::layer(const SString & nm) {
      return layerTable.find(nm) ;
   } ;

   //----------------------------------------------------------------------
   // new link

   //--------------------------------------------------
   // by StrForm

   BpLLink * BpNetBase::newLLink(StrForm & sform) {
      BpLLink * link = new BpLLink ;
      link->scan(sform,&layerTable,&linkTable) ;
      addLLink(link) ;
      return link ;
   } ;

   //--------------------------------------------------
   // by from/to layer

   BpLLink * BpNetBase::newLLink(BpLayer * from, BpLayer * to) {
      BpLLink * link = new BpLLink(from,to) ;
      addLLink(link) ;
      return link ;
   } ;

   //----------------------------------------------------------------------
   // add link

   void BpNetBase::addLLink(BpLLink * link) {
      linkTable.assign(link) ;
      link->registerFromTo() ;
   } ;
      
   //----------------------------------------------------------------------
   // get link

   BpLLink * BpNetBase::link(const SString & from, const SString & to) {
      return linkTable.find(from,to) ;
   } ;

   BpLLink * BpNetBase::link(BpLayer * from, BpLayer * to) {
      return linkTable.find(from,to) ;
   } ;


   //----------------------------------------------------------------------
   // randomize threshold and weight

   void BpNetBase::randomize(const Val & minval, const Val & maxval) {
      for(BpLayerTable::Iterator lyr = layerTable.begin() ;
	  lyr < layerTable.end() ;  lyr++) {
	 lyr.content()->randomizeThreshold(minval,maxval) ;
      }
      for(BpLLinkTable::Iterator lnk = linkTable.begin() ;
	  lnk < linkTable.end() ; lnk++) {
	 lnk.content()->randomizeWeight(minval,maxval) ;
      } 
   } 

   //======================================================================
   // I/O

   //----------------------------------------------------------------------
   // save

   void BpNetBase::save(ostream & ostr, 
			const Bool & newlinep,
			const SubString & indent) {
      SubString indent2 = indent + "  " ;
      SubString indent3 = indent2 + "  " ;
      StrForm::writeBeginTerm(ostr,sformTermHead()) ;

      StrForm::writeAssocRole(ostr,sformNameLbl) ; 
      StrForm::writeString(ostr,name) ;
      StrForm::writeArgSep(ostr) ;

      ostr << endl << indent2 ;

      StrForm::writeAssocRole(ostr,sformTempLbl) ; 
      ostr << NNN_VAL_FORMAT << temp ;
      StrForm::writeArgSep(ostr) ;

      StrForm::writeAssocRole(ostr,sformEpsLbl) ; 
      ostr << NNN_VAL_FORMAT << eps ;
      StrForm::writeArgSep(ostr) ;

      StrForm::writeAssocRole(ostr,sformAlphaLbl) ; 
      ostr << NNN_VAL_FORMAT << alpha ;
      StrForm::writeArgSep(ostr) ;

      ostr << endl << indent2 ;
      StrForm::writeAssocRole(ostr,sformLayerLbl) ; {
	 StrForm::writeBeginTerm(ostr) ;
	 StrForm::writeBeginList(ostr) ;
	 for(BpLayerTable::Iterator i = layerTable.begin() ; 
	     i < layerTable.end() ; i++) {
	    if(!layerTable.isBegin(i)) StrForm::writeArgSep(ostr) ;
	    ostr << endl << indent3 ;
	    i.content()->save(ostr,False,indent3) ;
	 }
	 ostr << endl << indent2 ;
	 StrForm::writeEndTerm(ostr) ;
      }
      StrForm::writeArgSep(ostr) ;

      ostr << endl << indent2 ;
      StrForm::writeAssocRole(ostr,sformLLinkLbl) ; {
	 StrForm::writeBeginTerm(ostr) ;
	 StrForm::writeBeginList(ostr) ;
	 for(BpLLinkTable::Iterator i = linkTable.begin() ; 
	     i < linkTable.end() ; i++) {
	    if(!linkTable.isBegin(i)) StrForm::writeArgSep(ostr) ;
	    ostr << endl << indent3 ;
	    i.content()->save(ostr,False,indent3) ;
	 }
	 ostr << endl << indent2 ;
	 StrForm::writeEndTerm(ostr) ;
      }
      StrForm::writeArgSep(ostr) ;

      ostr << endl << indent ;
      StrForm::writeEndTerm(ostr) ;
      if(newlinep) ostr << endl ;
   }
   
   //----------------------------------------------------------------------
   // scan

   void BpNetBase::scan(StrForm & sform) { 
      if(!sform.isTerm(sformTermHead())) {
	 ITK_ERR("Illegal BpLayer StrForm:" << sform) ;
	 throw 0 ;
      } 

      sform.assocCvtSString(sformNameLbl,name) ;

      sform.assocCvtFlt(sformTempLbl,temp) ;
      sform.assocCvtFlt(sformEpsLbl,eps) ;
      sform.assocCvtFlt(sformAlphaLbl,alpha) ;
      

      StrForm * layerlist = sform.assoc(sformLayerLbl) ;
      if(!isNull(layerlist)) {
	 for(Index i = 0 ; i < layerlist->lengthListArg() ; i++) {
	    newLayer(*(layerlist->nthListArg(i))) ;
	 } ;
      }

      StrForm * linklist = sform.assoc(sformLLinkLbl) ;
      if(!isNull(linklist)) {
	 for(Index i = 0 ; i < linklist->lengthListArg() ; i++) {
	    newLLink(*(linklist->nthListArg(i))) ;
	 } ;
      }
   } ;

} ;
