// -*- Mode:C++ -*-
//Header:
//File: BpNetBase.h
//Author: NODA, Itsuki
//Date: 1999/12/31
//

//ModifyHistory:
// 1999/12/31: Start to create this file
//EndModifyHistory:

#ifndef _Nnn_BpNetBase_h_
#define _Nnn_BpNetBase_h_
////////////////////////////////////////////////////////////////////////

#include <fstream.h>
#include "n3btype.h"
#include "n3utility.h"
#include "StrForm.h"
#include "BpLayer.h"
#include "BpLLink.h"

namespace Nnn {

   using namespace Itk ;

   //======================================================================
   // BpNetBase

   class BpNetBase : public WithDescriber {

      public:
	 static char* const _sformTermHead = "BpNetBase" ; // used in save/load
	 virtual char* sformTermHead() const {
	    return _sformTermHead ;
	 } ;
	 static char* const sformNameLbl = "name" ; // used in save/load
	 static char* const sformLayerLbl = "layer" ; // used in save/load
	 static char* const sformLLinkLbl = "llink" ; // used in save/load
	 static char* const sformTempLbl = "temp" ;
	 static char* const sformEpsLbl = "eps" ;
	 static char* const sformAlphaLbl = "alpha" ;

#define NNN_DFLT_TEMP	1.0
#define NNN_DFLT_EPS	0.01
#define NNN_DFLT_ALPHA	0.5

	 //--------------------------------------------------
	 // member
      public:
	 SString name ;
	 Val temp ;	// temperature of activation function
	 Val eps ;	// step of learning
	 Val alpha ;	// inertia param for learning
	 BpLayerTable layerTable ;
	 BpLLinkTable linkTable ;

	 //--------------------------------------------------
	 // constructor
      public:
	 BpNetBase() { init() ; } ;
	 BpNetBase(const SString & nm,
		   const Val & tempval = NNN_DFLT_TEMP,
		   const Val & epsval = NNN_DFLT_EPS,
		   const Val & alphaval = NNN_DFLT_ALPHA) { 
	    init(nm,tempval,epsval,alphaval) ; } ;

	 //--------------------------------------------------
	 // initialize
      public:
	 void init(const SString & nm = SString(),
		   const Val & tempval = NNN_DFLT_TEMP,
		   const Val & epsval = NNN_DFLT_EPS,
		   const Val & alphaval = NNN_DFLT_ALPHA) { 
	    name.copy(nm,True) ;
	    temp = tempval ;
	    eps = epsval ;
	    alpha = alphaval ;
	 } ;

	 //--------------------------------------------------
	 // new layer/link
      public:
	 BpLayer * newLayer(StrForm & sform) ;
	 BpLayer * newLayer(const SString & nm, UInt size = 0, 
			    BpLayer::FuncType ftype = BpLayer::FT_sigmoid) ;

	 BpLLink * newLLink(StrForm & sform) ;
	 BpLLink * newLLink(BpLayer * from, BpLayer * to) ;

	 //--------------------------------------------------
	 // add layer/link
      public:
	 void addLayer(BpLayer * layer) ;
	 void addLLink(BpLLink * link) ;

	 //--------------------------------------------------
	 // access
      public:
	 BpLayer * layer(const SString & nm) ;

	 BpLLink * link(const SString & from, const SString & to) ;
	 BpLLink * link(BpLayer * from, BpLayer * to) ;

	 BpLayer * findLayer(const SString & nm) { return layer(nm) ; } ;
	 BpLLink * findLink(const SString & from, const SString & to) {
	    return link(from,to) ;
	 } ;
	 BpLLink * findLink(BpLayer * from, BpLayer * to) {
	    return link(from,to) ;
	 } ;

	 //--------------------------------------------------
	 // randomize threshold and weight
      public:
	 void randomize(const Val & minval, const Val & maxval) ;

	 //--------------------------------------------------
	 // save 
      public:
	 virtual void save(const SubString& file) {
	    ofstream ostr(file) ;
	    return save(ostr) ;
	 } ;
	 virtual void save(ostream & ostr, 
			   const Bool & newlinep = True,
			   const SubString & indent = SubString("")) ;

	 //--------------------------------------------------
	 // load
      public:
	 virtual void load(const SubString & file) {
	    ifstream istr(file) ;
	    return load(istr) ;
	 } ;
	 virtual void load(istream & istr){
 	    StrForm * sform = StrForm::scan(istr) ;
	    scan(*sform) ;
	    delete sform ;
	 } ;


	 //--------------------------------------------------
	 // scan
      public:
	 virtual void scan(StrForm & sform) ;

	 //--------------------------------------------------
	 // describe
      public:
	 virtual void describe(ostream & ostr, 
			       const Bool detailp = True) const {
	    ostr << "#" << sformTermHead() << "[" << name << "]" ;

	    if(detailp) {
	       ostr << endl ;
	       ostr << "  " << "temp=" << NNN_VAL_FORMAT << temp << endl ;
	       ostr << "  " << "eps=" << NNN_VAL_FORMAT << eps << endl ;
	       ostr << "  " << "alpha=" << NNN_VAL_FORMAT << alpha << endl ;
	       ostr << "  " << "layer=" ;  layerTable.describe(ostr,True) ;
	       ostr << "  " << "llink=" ;  linkTable.describe(ostr,True) ;
	    } 
	 } ;
   } ;

} ;
////////////////////////////////////////////////////////////////////////
#endif


