// -*- Mode:C++ -*-
//Header:
//File: BpUnitBase.h
//Author: NODA, Itsuki
//Date: 1999/12/16
//

//ModifyHistory:
// 1999/12/16: Start to create this file
// 1999/12/26: reconfigulation
//EndModifyHistory:

#ifndef _Nnn_BpUnitBase_h_
#define _Nnn_BpUnitBase_h_
////////////////////////////////////////////////////////////////////////

#include "n3btype.h"
#include "n3utility.h"
#include "StrForm.h"
#include "UnitBase.h"

namespace Nnn {

   using namespace Itk ;

   //======================================================================
   // BpUnitBase

   class BpUnitBase : public UnitBase {

      public:
	 static char* const sformTermHead = "BpU" ; // used in save/load
	 static char* const sformThrLbl = "h" ;  // used in save/load
	 //--------------------------------------------------
	 // member
      public:
	 Val err ; // = delta-in
	 Val dlt ;
	 Val dh ;

	 //--------------------------------------------------
	 // constructor
      public:
	 BpUnitBase() : UnitBase() { err = 0.0 ; dlt = 0.0 ; dh = 0.0 ; } ;

	 //--------------------------------------------------
	 // forward processing
      public:
	 void activateByLinear(const Val & temp) {
	    out = inp / temp ;
	 } ;
	 void activateBySigmoid(const Val & temp) { 
	    out = sigmoid(inp,temp) ; 
	 } ;
	 void activateByBigmoid(const Val & temp) { 
	    out = bigmoid(inp,temp) ; 
	 } ;

	 //--------------------------------------------------
	 // backward processing
      public:
	 void calcError(const Val & tch) {
	    err = tch - out ;
	 } ;
	 Flt sqerr() const { return sq(err) ; } ;

	 void calcDeltaByLinear(const Val & temp) {
	    dlt = err / temp ;
	 } ;
	 void calcDeltaBySigmoid(const Val & temp) {
	    dlt = err * d_sigmoid(inp,out,temp) ;
	 } ;
	 void calcDeltaByBigmoid(const Val & temp) {
	    dlt = err * d_bigmoid(inp,out,temp) ;
	 } ;

	 //--------------------------------------------------
	 // save 
      public:
	 virtual void save(ostream & ostr) const ;

	 //--------------------------------------------------
	 // load
      public:
	 virtual void scan(StrForm & sform) ;

	 //--------------------------------------------------
	 // describe
      public:
	 virtual void describe(ostream & ostr, 
			       const Bool detailp = True) const {
	    ostr << "#BpUnitBase[" 
		 << "i=" << NNN_VAL_FORMAT << inp 
		 << ", h=" << NNN_VAL_FORMAT << thr 
		 << ", o=" << NNN_VAL_FORMAT << out 
		 << "]" ;

	    if(detailp) {
	       ostr << endl ;
	       ostr << "\t" << "err=" << NNN_VAL_FORMAT << err << endl ;
	       ostr << "\t" << "dlt=" << NNN_VAL_FORMAT << dlt << endl ;
	       ostr << "\t" << "dh=" << NNN_VAL_FORMAT << dh << endl ;
	    } 
	 } ;
   } ;

} ;
////////////////////////////////////////////////////////////////////////
#endif
