// -*- Mode:C++ -*-
//Header:
//File: StrForm.cc
//Author: NODA, Itsuki
//Date: 1999/12/23
//

//ModifyHistory:
// 1999/12/23: Start to create this file
//EndModifyHistory:

//StrForm Syntax:
//	<Form> ::= <Atom> | <Term>
//	<Atom> ::= <Number_Symbol_Atom> | <String>
//	<Number_Symbol_Atom> ::= that does not include separators
//	<Symbol> ::= `"' (any char string except `"'(escape = `/')) `"'
//	<Term> ::= `#' <Head> `[' <AssocArgs> [ `|' <ListArgs> ] `]'

#include "StrForm.h"

namespace Itk {

   //======================================================================
   // StrForm class

   //----------------------------------------------------------------------
   // stream skipping utility

   //--------------------------------------------------
   // chkEndError

   void StrForm::chkEndError(istream & istr) {
      if(istr.eof()) {
	 ITK_ERR("stream expired during read.") ;
	 throw 0  ;
      } 
   }

   //--------------------------------------------------
   // skipSpace

   UInt StrForm::skipSpace(istream & istr) {
      UInt n = 0 ;
      while(True) {
	 chkEndError(istr) ;
	 if(!isspace(istr.peek())) return n ;
	 istr.get() ;
	 n++ ;
      }
      return 0 ; /* never reach here */
   } ;

   //--------------------------------------------------
   // skip comments

   UInt StrForm::skipComment(istream & istr) {
      UInt n = 0 ;
      if( istr.peek() != ITK_SST_COMMENT_BEGIN ) return n ;
      while(True) {
	 chkEndError(istr) ;
	 n++ ;
	 if( istr.get() == ITK_SST_COMMENT_END ) return n ;
      }
      return 0 ; /* never reach here */
   }

   //--------------------------------------------------
   // skip space & comment
   
   UInt StrForm::skipSpaceComment(istream & istr) {
      UInt n = skipSpace(istr) ;
      UInt m = skipComment(istr) ;
      UInt l = 0;
      if(m > 0) {
	 l = skipSpaceComment(istr) ;
      } ;
      return n + m + l ;
   } 
	 
   //----------------------------------------------------------------------
   // prescan and make Buffer

   //--------------------------------------------------
   // toplevel

   Buffer StrForm::prescan(istream & istr) {
      Buffer * buf = new Buffer(ITK_SST_INITBUFSIZE) ;
      prescanTop(*buf,istr) ;
      return *buf ;
   } ;

   //--------------------------------------------------
   // prescan Top

   Bool StrForm::prescanTop(Buffer & buf, istream & istr) {
      skipSpaceComment(istr) ;
      switch (istr.peek()) {
	 case ITK_SST_TERM_PREFIX :  return prescanTerm(buf,istr) ;
	 case ITK_SST_STRING_BEGIN : return prescanString(buf,istr) ;
	 default :                   return prescanAtom(buf,istr) ;
      }
   }

   //--------------------------------------------------
   // scan Atom

   Bool StrForm::prescanAtom(Buffer & buf, istream & istr) {
      skipSpaceComment(istr) ;
      Bool r = False ;
      while(True) {
	 if(buf.nullp() || index(ITK_SST_NONATOMCHAR,istr.peek())) 
	    return r ;
	 buf.putCharSafe(istr.get()) ;
	 r = True ;
     }
      return False ; /* never reach here */
   }

   //--------------------------------------------------
   // scan string
   
   Bool StrForm::prescanString(Buffer & buf, istream & istr) {
      skipSpaceComment(istr) ;
      if(istr.peek() != ITK_SST_STRING_BEGIN) return False ;
      buf.putCharSafe(istr.get()) ;
      while(True) {
	 chkEndError(istr) ;
	 char c = buf.putCharSafe(istr.get()) ;
	 if(c == ITK_SST_STRING_END) {
	    return True ;
	 } else if (c == ITK_SST_STRING_ESCAPE) {
	    chkEndError(istr) ;
	    buf.putCharSafe(istr.get()) ;
	 } 
      }
      return False ; /* never reach here */
   }

   //--------------------------------------------------
   // scan Term

   Bool StrForm::prescanTerm(Buffer & buf, istream & istr) {
      skipSpaceComment(istr) ;
      if(istr.peek() != ITK_SST_TERM_PREFIX) return False ;
      
      buf.putCharSafe(istr.get()) ;

      prescanAtom(buf,istr) ; /* scan head part */

      prescanTermAssocArg(buf,istr) ; /* scan assoc arg part */

      prescanTermListArg(buf,istr) ; /* scan list arg part */
      return True ;
   }

   //--------------------------------------------------
   // scan Term AssocArg

   Bool StrForm::prescanTermAssocArg(Buffer & buf, istream & istr) {
      skipSpaceComment(istr) ;
      if(buf.putCharSafe(istr.get()) != ITK_SST_TERM_BEGIN) {
	 ITK_ERR("Illegal Term Syntax.") ; 
 	 cerr << "   buffer=" << buf ;
	 throw 0 ; 
      } 
      return prescanTermAssocArgSub(buf,istr) ;
   } ;

   //--------------------------------------------------
   // scan Term AssocArgSub

   Bool StrForm::prescanTermAssocArgSub(Buffer & buf, istream & istr) {
      if(!prescanAtom(buf,istr)) return False ;

      skipSpaceComment(istr) ;
      if(buf.putCharSafe(istr.get()) != ITK_SST_TERM_ASSOCOP) {
	 ITK_ERR("Illegal AssocArg Syntax.") ; 
 	 cerr << "   buffer=" << buf ;
	 throw 0 ;
      } 

      if(!prescanTop(buf,istr)) {
	 ITK_ERR("Illegal AssocArg Syntax.") ; 
 	 cerr << "   buffer=" << buf ;
	 throw 0 ;
      }

      skipSpaceComment(istr) ;
      if(istr.peek() == ITK_SST_TERM_SEP) {
	 buf.putCharSafe(istr.get()) ;
	 prescanTermAssocArgSub(buf,istr) ;
      }

      return True ;
   } 

   //--------------------------------------------------
   // scan Term ListArg

   Bool StrForm::prescanTermListArg(Buffer & buf, istream & istr) {
      skipSpaceComment(istr) ;
      if(istr.peek() == ITK_SST_TERM_END) {
	 buf.putCharSafe(istr.get()) ;
	 return False ;
      }
      if(buf.putCharSafe(istr.get()) != ITK_SST_TERM_LISTARGSEP) {
	 ITK_ERR("Illegal Term Syntax.") ; 
 	 cerr << "   buffer=" << buf ;
	 throw 0 ;
      }
      prescanTermListArgSub(buf,istr) ;
      if(buf.putCharSafe(istr.get()) != ITK_SST_TERM_END) {
	 ITK_ERR("Illegal Term Syntax.") ; 
 	 cerr << "   buffer=" << buf ;
	 throw 0 ;
      }
      return True ;
   } 

   //--------------------------------------------------
   // scan Term ListArg Sub

   Bool StrForm::prescanTermListArgSub(Buffer & buf, istream & istr) {
      if(prescanTop(buf,istr)) {
	 skipSpaceComment(istr) ;
	 if(istr.peek() == ITK_SST_TERM_SEP) {
	    buf.putCharSafe(istr.get()) ;
	    prescanTermListArgSub(buf,istr) ;
	 }
      }
      return True ;
   } 

   //----------------------------------------------------------------------
   // buffer skipping utility

   //--------------------------------------------------
   // chkEndError

   void StrForm::chkEndError(Buffer & buf) {
      if(buf.nullp()) {
	 ITK_ERR("buffer expired during read.") ;
 	 cerr << "   buffer=" << buf ;
	 throw 0 ;
      } 
   }

   //--------------------------------------------------
   // skipSpace

   UInt StrForm::skipSpace(Buffer & buf) {
      UInt n = 0 ;
      while(True) {
	 chkEndError(buf) ;
	 if(!isspace(buf.first())) return n ;
	 buf.ignoreChar() ;
	 n++ ;
      }
      return 0 ; /* never reach here */
   } ;

   //--------------------------------------------------
   // skip comments

   UInt StrForm::skipComment(Buffer & buf) {
      UInt n = 0 ;
      if( buf.first() != ITK_SST_COMMENT_BEGIN ) return n ;
      while(True) {
	 chkEndError(buf) ;
	 n++ ;
	 if( buf.getChar() == ITK_SST_COMMENT_END ) return n ;
      }
      return 0 ; /* never reach here */
   }

   //--------------------------------------------------
   // skip space & comment
   
   UInt StrForm::skipSpaceComment(Buffer & buf) {
      UInt n = skipSpace(buf) ;
      UInt m = skipComment(buf) ;
      UInt l = 0;
      if(m > 0) {
	 l = skipSpaceComment(buf) ;
      } ;
      return n + m + l ;
   } 
	 
   //----------------------------------------------------------------------
   // scan SubString

   //--------------------------------------------------
   // scan 

   StrForm * StrForm::scan(Buffer & buf) {
      return scanTop(buf) ;
   } 

   //--------------------------------------------------
   // scan Top

   StrForm * StrForm::scanTop(Buffer & buf) {
      skipSpaceComment(buf) ;
      switch (buf.first()) {
	 case ITK_SST_TERM_PREFIX :  return scanTerm(buf) ;
	 case ITK_SST_STRING_BEGIN : return scanString(buf) ;
	 default :                   return scanAtom(buf) ;
      }
   }

   //--------------------------------------------------
   // scan Atom

   StrForm * StrForm::scanAtom(Buffer & buf) {
      skipSpaceComment(buf) ;
      char * originalhead = buf.head ;
      while(True) {
	 if(index(ITK_SST_NONATOMCHAR,buf.first()) || buf.nullp()) {
	    if (originalhead == buf.head) return ITK_NULLPTR ;
	    StrForm * s = new StrForm ;
	    s->type = Type_Atom ;
	    s->str.head = originalhead ;
	    s->str.tail = buf.head ;
	    return s ;
	 } 
	 buf.ignoreChar() ;
      }
      return ITK_NULLPTR ; /* never reach here */
   }

   //--------------------------------------------------
   // scan string
   
   StrForm * StrForm::scanString(Buffer & buf) {
      skipSpaceComment(buf) ;
      if(buf.first() != ITK_SST_STRING_BEGIN) return ITK_NULLPTR ;
      char * originalhead = buf.head ;
      buf.ignoreChar() ;
      while(True) {
	 chkEndError(buf) ;
	 char c = buf.getChar() ;
	 if(c == ITK_SST_STRING_END) {
	    StrForm * s = new StrForm ;
	    s->type = Type_Atom ;
	    s->str.head = originalhead ;
	    s->str.tail = buf.head ;
	    s->head.head = s->str.head+1 ;
	    s->head.tail = s->str.tail-1 ;
	    return s ;
	 } else if (c == ITK_SST_STRING_ESCAPE) {
	    chkEndError(buf) ;
	    buf.ignoreChar() ;
	 } 
      }
      return ITK_NULLPTR ; /* never reach here */
   }

   //--------------------------------------------------
   // scan Term

   StrForm * StrForm::scanTerm(Buffer & buf) {
      skipSpaceComment(buf) ;
      if(buf.first() != ITK_SST_TERM_PREFIX) return ITK_NULLPTR ;
      
      StrForm * s = new StrForm ;
      s->type = Type_Term ;
      s->str.head = buf.head ;
      buf.ignoreChar() ;

      /* scan head part */
      
      StrForm * head = scanAtom(buf) ;
      if(isNull(head)) {
	 s->head.set(buf.head,buf.head) ;
      } else {
	 s->head.set(head->str) ;
	 delete head ;
      }

      /* scan assoc arg part */
      s->assocArg = scanTermAssocArg(buf) ;

      /* scan list arg part */
      s->listArg = scanTermListArg(buf) ;

      s->str.tail = buf.head ;

      s->next = ITK_NULLPTR ;

      return s ;
   }

   //--------------------------------------------------
   // scan Term AssocArg

   StrForm::AssocArg * StrForm::scanTermAssocArg(Buffer & buf) {
      skipSpaceComment(buf) ;
      if(buf.getChar() != ITK_SST_TERM_BEGIN) {
	 ITK_ERR("Illegal Term Syntax.") ; 
 	 cerr << "   buffer=" << buf ;
	 throw 0 ; 
      } 
      return scanTermAssocArgSub(buf) ;
   } ;

   //--------------------------------------------------
   // scan Term AssocArgSub

   StrForm::AssocArg * StrForm::scanTermAssocArgSub(Buffer & buf) {
      StrForm * role = scanAtom(buf) ;
      if(isNull(role)) return ITK_NULLPTR ;

      AssocArg * arg = new AssocArg ;
      arg->role.set(role->str) ;
      delete role ;

      skipSpaceComment(buf) ;
      if(buf.getChar() != ITK_SST_TERM_ASSOCOP) {
	 ITK_ERR("Illegal AssocArg Syntax.") ; 
 	 cerr << "   buffer=" << buf ;
	 throw 0 ;
      } 

      arg->value = scanTop(buf) ;
      
      if(isNull(arg->value)) {
	 ITK_ERR("Illegal AssocArg Syntax.") ; 
 	 cerr << "   buffer=" << buf ;
	 throw 0 ;
      }

      skipSpaceComment(buf) ;
      arg->next = ITK_NULLPTR ;
      if(buf.first() == ITK_SST_TERM_SEP) {
	 buf.ignoreChar() ;
	 arg->next = scanTermAssocArgSub(buf) ;
      }

      return arg ;
   } 

   //--------------------------------------------------
   // scan Term ListArg

   StrForm * StrForm::scanTermListArg(Buffer & buf) {
      skipSpaceComment(buf) ;
      if(buf.first() == ITK_SST_TERM_END) {
	 buf.getChar() ;
	 return ITK_NULLPTR ;
      }
      if(buf.getChar() != ITK_SST_TERM_LISTARGSEP) {
	 ITK_ERR("Illegal Term Syntax.") ; 
 	 cerr << "   buffer=" << buf ;
	 throw 0 ;
      }
      StrForm * r = scanTermListArgSub(buf) ;
      if(buf.getChar() != ITK_SST_TERM_END) {
	 ITK_ERR("Illegal Term Syntax.") ; 
 	 cerr << "   buffer=" << buf ;
	 throw 0 ;
      }
      return r ;
   } 

   //--------------------------------------------------
   // scan Term ListArg Sub

   StrForm * StrForm::scanTermListArgSub(Buffer & buf) {
      StrForm * arg = scanTop(buf) ;
      if(!isNull(arg)) {
	 skipSpaceComment(buf) ;
	 if(buf.first() == ITK_SST_TERM_SEP) {
	    buf.ignoreChar() ;
	    arg->next = scanTermListArgSub(buf) ;
	 }
      }
      return arg ;
   } 

   //----------------------------------------------------------------------
   // describe

   void StrForm::describe(ostream & ostr, const Bool detailp) const {
      if(detailp) {
	 ostr << "#StrForm[" ;
	 if(type == Type_Atom) {
	    ostr << "type=atom, data=" << str ;
	 } else {
	    ostr << "type=term, head=" << head ;
	 }
	 ostr << "]" << endl ;
	 if(type == Type_Term) {
	    ostr << "  " << "assocArg=(" ;
	    for( AssocArg * arg = assocArg ; 
		 !isNull(arg) ; arg = arg->next ) {
	       if(arg != assocArg) ostr << ", " ;
	       ostr << *arg ;
	    } 
	    ostr << ")" << endl ;
	    ostr << "  " << "listArg=(" ;
	    for( StrForm * arg = listArg ; 
		 !isNull(arg) ; arg = arg->next ) {
	       if(arg != listArg) ostr << ", " ;
	       ostr << *arg ;
	    }
	    ostr << ")" << endl ;
	 }
      } else {
	 if(type == Type_Atom) {
	    ostr << str ;
	 } else {
	    ostr << ITK_SST_TERM_PREFIX << head << ITK_SST_TERM_BEGIN ;
	    for( AssocArg * arg = assocArg ;
		 !isNull(arg) ; arg = arg->next ) {
	       if(arg != assocArg) ostr << ITK_SST_TERM_SEP << " " ;
	       ostr << arg->role << ITK_SST_TERM_ASSOCOP << *arg->value ;
	    }
	    if(!isNull(listArg)) {
	       ostr << ITK_SST_TERM_LISTARGSEP ;
	       for( StrForm * arg = listArg ;
		    !isNull(arg) ; arg = arg->next ) {
		  if(arg != listArg) ostr << ITK_SST_TERM_SEP << " " ;
		  ostr << *arg ;
	       }
	    }
	    ostr << ITK_SST_TERM_END ;
	 }
      }
   }
} ;
