// -*- Mode:C++ -*-
//Header:
//File: UnitBase.h
//Author: NODA, Itsuki
//Date: 1999/12/16
//

//ModifyHistory:
// 1999/12/16: Start to create this file
// 1999/12/26: reconfigulation
//EndModifyHistory:

#ifndef _Nnn_UnitBase_h_
#define _Nnn_UnitBase_h_
////////////////////////////////////////////////////////////////////////

#include "n3btype.h"
#include "n3utility.h"
#include "StrForm.h"

namespace Nnn {

   using namespace Itk ;

   //======================================================================
   // UnitBase

   class UnitBase : public WithDescriber {

      public:
	 static char* const sformTermHead = "U" ; // used in save/load
	 static char* const sformThrLbl = "h" ;  // used in save/load

	 //--------------------------------------------------
	 // member
      public:
	 Val inp ;
	 Val thr ;
	 Val out ;

	 //--------------------------------------------------
	 // constructor
      public:
	 UnitBase() { inp = 0.0 ; out = 0.0 ; thr = 0.0 ; } ;
	 UnitBase(const Val & h) { 
	    inp = 0.0 ; out = 0.0 ; thr = h ; 
	 } ;
	 UnitBase(const Val& i, const Val& o, const Val & h) { 
	    inp = i ; out = o ; thr = h ;
	 } ;
	 UnitBase(const UnitBase& u) { 
	    inp = u.inp ; out = u.out ; thr = u.thr ;
	 } ;

	 //--------------------------------------------------
	 // clear input
      public:
	 void clearInput(const Val& v = 0.0) { inp = v ; } ;

	 //--------------------------------------------------
	 // clearToThreshold (reset input to threshold)
      public:
	 void clearInputToThreshold() { clearInput(thr) ; } ;

	 //--------------------------------------------------
	 // save 
      public:
	 virtual void save(ostream & ostr) const ;

	 //--------------------------------------------------
	 // load
      public:
	 virtual void load(istream & istr) {
	    StrForm * sform = StrForm::scan(istr) ;
	    scan(*sform) ;
	    delete sform ;
	 } ;

      public:
	 virtual void scan(StrForm & sform) ;

	 //--------------------------------------------------
	 // describe
      public:
	 virtual void describe(ostream & ostr, 
			       const Bool detailp = True) const {
	    ostr << "#UnitBase[" 
		 << "i=" << NNN_VAL_FORMAT << inp 
		 << "/h=" << NNN_VAL_FORMAT << thr 
		 << "/o=" << NNN_VAL_FORMAT << out 
		 << "]" ;

	    if(detailp) {
	       ostr << endl ;
	    } 
	 } ;
   } ;

} ;
////////////////////////////////////////////////////////////////////////
#endif
