// -*- Mode:C++ -*-
//Header:
//File: VElman5Lb.h
//Title: VElman Net with 5 sigmoid layers with bypass
//Author: NODA, Itsuki
//Date: 2000/10/23
//

//ModifyHistory:
// 2000/01/05: Start to create this file (originally in VElman5L.h)
// 2000/10/23: Copy from VElman5L.h
//EndModifyHistory:

//Memo:
//	structure:
//			[output]
//			 |    |
//			 | {inter2}
//			 |    |
//			[hidden______]
//			 |    |     | 
//			 | {inter1} |
//			 |    |  |  |
//			[input] [context]
//
//EndMemo:

#ifndef _Nnn_VElman5Lb_h_
#define _Nnn_VElman5Lb_h_
////////////////////////////////////////////////////////////////////////

#include "VBpNetBase.h"
#include "TrainSet4XpdVElman5Lb.h"

namespace Nnn {

   using namespace Itk ;

   //======================================================================
   // VElman 5 Layer with bypass

   class VElman5Lb : public VBpNetBase {

      public:
	 //--------------------------------------------------
	 // constants

	 static char * const nameContext = "context" ;
	 static char * const nameInput   = "input" ;
	 static char * const nameInter1  = "inter1" ;
	 static char * const nameHidden  = "hidden" ;
	 static char * const nameInter2  = "inter2" ;
	 static char * const nameOutput  = "output" ;

	 //--------------------------------------------------
	 // member
      public:
	 VBpLayer * context ;
	 VBpLayer * input ;
	 VBpLayer * inter1 ;
	 VBpLayer * hidden ;
	 VBpLayer * inter2 ;
	 VBpLayer * output ;

	 VBpLLink * con_int1 ;
	 VBpLLink * inp_int1 ;
	 VBpLLink * con_hid ;
	 VBpLLink * inp_hid ;
	 VBpLLink * int1_hid ;
	 VBpLLink * hid_int2 ;
	 VBpLLink * hid_out ;
	 VBpLLink * int2_out ;

	 //--------------------------------------------------
	 // constructor
      public:
	 VElman5Lb() { init() ; } ;

	 //--------------------------------------------------
	 // alloc new memory
     public:
	 void alloc(const SString & nm,
		    const UInt & inpsz,
		    const UInt & int1sz,
		    const UInt & hidsz,
		    const UInt & int2sz,
		    const UInt & outsz,
		    const Val & tempval = NNN_DFLT_TEMP,
		    const Val & epsval = NNN_DFLT_EPS,
		    const Val & alphaval = NNN_DFLT_ALPHA) {
	    init(nm,tempval,epsval,alphaval) ;
	    assignLayerLLink(inpsz,int1sz,hidsz,int2sz,outsz) ;
	 } ;
	 
	 //--------------------------------------------------
	 // assignLayer (l-s-l-s-l)
      public:
	 void assignLayerLLink(const UInt & inpsz, 
			       const UInt & int1sz,
			       const UInt & hidsz, 
			       const UInt & int2sz,
			       const UInt & outsz) {

	    context = newLayer(nameContext, hidsz,  VBpLayer::FT_linear) ;
	    input   = newLayer(nameInput,   inpsz,  VBpLayer::FT_linear) ;
	    inter1  = newLayer(nameInter1,  int1sz, VBpLayer::FT_sigmoid) ;
	    hidden  = newLayer(nameHidden,  hidsz,  VBpLayer::FT_linear) ;
	    inter2  = newLayer(nameInter2,  int2sz, VBpLayer::FT_sigmoid) ;
	    output  = newLayer(nameOutput,  outsz,  VBpLayer::FT_linear) ;
	    
	    con_int1 = newLLink(context, inter1) ;
	    inp_int1 = newLLink(input,   inter1) ;
	    con_hid  = newLLink(context, hidden);
	    inp_hid  = newLLink(input,   hidden);
	    int1_hid = newLLink(inter1,  hidden);
	    hid_int2 = newLLink(hidden,  inter2) ;
	    hid_out  = newLLink(hidden,  output) ;
	    int2_out = newLLink(inter2,  output) ;
	 } ;

	 //--------------------------------------------------
	 // renewContext
      public:
	 void renewContext() {
	    context->setOut(hidden->out()) ;
	 } ;

	 //--------------------------------------------------
	 // propFore
      public:
	 void propFore(const ValVector & inpvec, Bool newcontextp = True) {
	    if(newcontextp) renewContext() ;

	    input->propFore(inpvec,temp) ;
	    
	    inter1->propFore(temp) ;
	    hidden->propFore(temp) ;
	    inter2->propFore(temp) ;
	    output->propFore(temp) ;
	 } ;
	    
	 //--------------------------------------------------
	 // propBack
      public:
	 void propBack(const ValVector & tchvec, Bool backtoinputp = False) {
	    output->propBack(tchvec,temp) ;
	    inter2->propBack(temp) ;
	    hidden->propBack(temp) ;
	    inter1->propBack(temp) ;
	    if(backtoinputp) {
	       input->propBack(temp) ;
	       context->propBack(temp) ;
	    }
	 } ;

	 //--------------------------------------------------
	 // propBack
      public:
	 void propBackWithMask(const ValVector & tchvec, 
			       const ValVector & tchmask,
			       Bool backtoinputp = False) {
	    output->propBackWithMask(tchvec,tchmask,temp) ;
	    inter2->propBack(temp) ;
	    hidden->propBack(temp) ;
	    inter1->propBack(temp) ;
	    if(backtoinputp) {
	       input->propBack(temp) ;
	       context->propBack(temp) ;
	    }
	 } ;

	 //--------------------------------------------------
	 // learn
      public:
	 Val learn() {
	    Val dsum = 0.0 ;
#if 0
	    /* 
	     *  this does not work for empty links
	     */
	    dsum += inter1->learn(eps,alpha) ;
	    dsum += hidden->learn(eps,alpha) ;
	    dsum += inter2->learn(eps,alpha) ;
	    dsum += output->learn(eps,alpha) ;

	    dsum += con_int1->learn(eps,alpha) ;
	    dsum += inp_int1->learn(eps,alpha) ;
	    dsum += con_hid->learn(eps,alpha) ;
	    dsum += inp_hid->learn(eps,alpha) ;
	    dsum += int1_hid->learn(eps,alpha) ;
	    dsum += hid_int2->learn(eps,alpha) ;
	    dsum += hid_out->learn(eps,alpha) ;
	    dsum += int2_out->learn(eps,alpha) ;
#endif
	    for(VBpLayerTable::Iterator i = layerTable.begin() ; 
		i < layerTable.end() ; i++) {
	       i.content()->learn(eps,alpha) ;
	    }

	    for(VBpLLinkTable::Iterator i = linkTable.begin() ; 
		i < linkTable.end() ; i++) {
	       i.content()->learn(eps,alpha) ;
	    }
	    
	    cntLearn++ ;

	    return dsum ;
	 } ;

	 //--------------------------------------------------
	 // run
      public:
	 void run(const ValVector & inpvec) { 
	    propFore(inpvec) ; 
	    cntTrial++ ;
	 } ;
	 Val runLearn(const ValVector & inpvec,
		       const ValVector & tchvec) {
	    propFore(inpvec) ;
	    cntTrial++ ;
	    propBack(tchvec) ;
	    return learn() ;
	 } ;

	 //--------------------------------------------------
	 // runAdapt
	 //
	 //     Z0 = X_inp
	 //     loop(i) {
	 //       Xi = mask_x * X_inp + (1-mask_x) * Zi
	 //       Yi = f(Xi)
	 //       Ei = mask_y * (Y_tch - Yi)
	 //       Di = Ei * f'(Xi)
	 //       Zi+1 = Xi + alpha * Di
	 //     }

      public:
	 void runAdapt(const ValVector & inpvec, const ValVector & inpmask,
		       const ValVector & tchvec, const ValVector & tchmask,
		       const Val & alpha, const UInt & n) {

	    renewContext() ;
	    input->rinp().copyFrom(inpvec) ;

	    for(UInt i = 0 ; i < n ; i++) {
	       input->propForeWithMask(inpvec,inpmask,temp) ;
	       inter1->propFore(temp) ;
	       hidden->propFore(temp) ;
	       inter2->propFore(temp) ;
	       output->propFore(temp) ;

	       output->propBackWithMask(tchvec,tchmask,temp) ;
	       inter2->propBack(temp) ;
	       hidden->propBack(temp) ;
	       inter1->propBack(temp) ;
	       input->propBack(temp) ;

	       input->adaptRInpByDelta(alpha) ;
	    }
	 } ;

	 //--------------------------------------------------
	 // runAdapt2

      public:
	 Flt runAdapt2(const ValVector & inpvec, const ValVector & inpmask,
		       const ValVector & tchvec, const ValVector & tchmask,
		       const ValVector & ctxmask,
		       const Val & alpha, const UInt & n,
		       const Bool learnp = False) {

	    //renewContext() ;

	    //thr is used to keep previous hidden
	    context->thr().copyFrom(hidden->out()) ;  
	    context->rinp().copyFrom(hidden->out()) ;

	    input->rinp().copyFrom(inpvec) ;

	    for(UInt i = 0 ; i < n ; i++) {
	       context->propForeWithMask(context->thr(),ctxmask,temp) ;
	       input->propForeWithMask(inpvec,inpmask,temp) ;
	       inter1->propFore(temp) ;
	       hidden->propFore(temp) ;
	       inter2->propFore(temp) ;
	       output->propFore(temp) ;

	       output->propBackWithMask(tchvec,tchmask,temp) ;
	       inter2->propBack(temp) ;
	       hidden->propBack(temp) ;
	       inter1->propBack(temp) ;
	       input->propBack(temp) ;
	       context->propBack(temp) ;

	       input->adaptRInpByDelta(alpha) ;
	       context->adaptRInpByDelta(alpha) ;
	    }
	    cntTrial++ ;

	    if(learnp) { return learn() ; }
	    else return 0.0 ;

	 } ;

	 //--------------------------------------------------
	 // get sum of square error
      public:
	 Flt sumError() const { return output->sumError() ; } ;

	 //--------------------------------------------------
	 // get output
      public:
	 void getNetOutput(ValVector & outvec) const { 
	    output->getNetOutput(outvec) ;
	 } ;

	 //--------------------------------------------------
	 // scan
      public:
	 virtual void scan(StrForm & sform) {
	    VBpNetBase::scan(sform) ;
	    assignNamedLayerLLink() ;
	 } ;

	 //--------------------------------------------------
	 // assignNamedLayerLLink
      public:
	 void assignNamedLayerLLink() {
	    context = layer(nameContext) ;
	    input   = layer(nameInput) ;
	    inter1  = layer(nameInter1) ;
	    hidden  = layer(nameHidden) ;
	    inter2  = layer(nameInter2) ;
	    output  = layer(nameOutput) ;

	    con_int1 = link(context, inter1) ;
	    inp_int1 = link(input,   inter1) ;
	    con_hid =  link(context, hidden) ;
	    inp_hid =  link(input,   hidden) ;
	    int1_hid = link(inter1,  hidden) ;
	    hid_int2 = link(hidden,  inter2) ;
	    hid_out =  link(hidden,  output) ;
	    int2_out = link(inter2,  output) ;
	 } ;
   } ;

} ;
////////////////////////////////////////////////////////////////////////
#endif


