// -*- Mode:C++ -*-
//Header:
//File: VElman5LbP.h
//Title: VElman Net with 5 sigmoid layers with bypass with Predict Layer
//Author: NODA, Itsuki
//Date: 2001/05/21
//

//ModifyHistory:
// 2000/01/05: Start to create this file (originally in VElman5L.h)
// 2000/10/23: Copy from VElman5L.h
// 2001/05/21: Copy from XpdVElman5Lb.h
//EndModifyHistory:

//Memo:
//	structure:
//                                  [output']  == nxoutput
//                                   |  |
//                                   | {inter2'} == nxinter2
//			             |  |
//			[output]    [hidden'] == nxhidden
//			 |    |      |  |
//			 | {inter2}  |{inter1'} == nxinter1
//			 |    |      |/
//			[hidden________]
//			 |    |     | 
//			 | {inter1} |
//			 |    |  |  |
//			[input] [context]
//
//EndMemo:


#ifndef _Nnn_VElman5LbP_h_
#define _Nnn_VElman5LbP_h_
////////////////////////////////////////////////////////////////////////

#include "VElman5Lb.h"
//#include "TrainSet4VElman5LbP.h"

namespace Nnn {

   using namespace Itk ;

   //======================================================================
   // VElman 5 Layer with bypass with predict layer

   class VElman5LbP : public VElman5Lb {

      public:
	 //--------------------------------------------------
	 // member

	 VElman5Lb * mother ;

	 VBpLayer * nxinter1 ;
	 VBpLayer * nxhidden ;
	 VBpLayer * nxinter2 ;
	 VBpLayer * nxoutput ;
	 
	 VBpLLink * nx_con_int1 ;
	 VBpLLink * nx_con_hid ;
	 VBpLLink * nx_int1_hid ;
	 VBpLLink * nx_hid_int2 ;
	 VBpLLink * nx_hid_out ;
	 VBpLLink * nx_int2_out ;

	 //--------------------------------------------------
	 // derive from normal VElman5Lb
      public:

	 void deriveFrom(VElman5Lb * elman) {
	    mother = elman ;

	    copyFrom(*elman,True,True) ;
	    assignNamedLayerLLink() ;
	 } ;

	 void assignNamedLayerLLink() {
	    VElman5Lb::assignNamedLayerLLink() ;

	    nxinter1 = derivePredLayer(inter1,"nxinter1") ;
	    nxhidden = derivePredLayer(hidden,"nxhidden") ;
	    nxinter2 = derivePredLayer(inter2,"nxinter2") ;
	    nxoutput = derivePredLayer(output,"nxoutput") ;

	    nx_con_int1 = derivePredLLink(con_int1,hidden,  nxinter1) ;
	    nx_con_hid  = derivePredLLink(con_hid, hidden,  nxhidden) ;
	    nx_int1_hid = derivePredLLink(int1_hid,nxinter1,nxhidden) ;
	    nx_hid_int2 = derivePredLLink(hid_int2,nxhidden,nxinter2) ;
	    nx_hid_out  = derivePredLLink(hid_out, nxhidden,nxoutput) ;
	    nx_int2_out = derivePredLLink(int2_out,nxinter2,nxoutput) ;
	    
	 } ;

	 //----------------------------------------
	 // derive layer and link for prediction
      public:
	 VBpLayer * derivePredLayer(VBpLayer * orig, const SString & name) {
	    VBpLayer * layer = &(orig->duplicate(True)) ;
	    layer->name = name ;
	    addLayer(layer) ;
	    return layer ;
	 } ;
	 
      public:
	 VBpLLink * derivePredLLink(VBpLLink * orig,
				    VBpLayer * from, VBpLayer * to) {
	    VBpLLink * link = new VBpLLink(from,to) ;
	    link->copyMiscParamFrom(*orig) ;
	    link->copyWeightFrom(*orig,True) ;
	    addLLink(link) ;
	    return link ;
	 } ;

	 //--------------------------------------------------
	 // propFore
      public:
	 void propFore(const ValVector & inpvec, Bool newcontextp = True) {
	    VElman5Lb::propFore(inpvec,newcontextp) ;
	    
	    nxinter1->propFore(temp) ;
	    nxhidden->propFore(temp) ;
	    nxinter2->propFore(temp) ;
	    nxoutput->propFore(temp) ;
	 } ;

	 //--------------------------------------------------
	 // propBack
      public:
	 void propBack(const ValVector & tchvec, 
		       const ValVector & nxtchvec,
		       Bool backtoinputp = False) {
	    nxoutput->propBack(nxtchvec,temp) ;
	    nxinter2->propBack(temp) ;
	    nxhidden->propBack(temp) ;
	    nxinter1->propBack(temp) ;

	    VElman5Lb::propBack(tchvec,backtoinputp) ;
	 } ;

	 //--------------------------------------------------
	 // propBack with Mask
      public:
	 void propBackWithMask(const ValVector & tchvec, 
			       const ValVector & tchmask,
			       const ValVector & nxtchvec,
			       const ValVector & nxtchmask,
			       Bool backtoinputp = False) {
	    nxoutput->propBackWithMask(nxtchvec,nxtchmask,temp) ;
	    nxinter2->propBack(temp) ;
	    nxhidden->propBack(temp) ;
	    nxinter1->propBack(temp) ;

	    VElman5Lb::propBackWithMask(tchvec,tchmask,backtoinputp) ;
	 } ;

	 //--------------------------------------------------
	 // learn
      public:
	 Val learn() {
	    mother->cntLearn++ ;
	    return VElman5Lb::learn() ;
	 } ;

	 //--------------------------------------------------
	 // run
      public:
	 void run(const ValVector & inpvec) { 
	    propFore(inpvec) ; 
	    cntTrial++ ;
	    mother->cntTrial++ ;
	 } ;
	 Val runLearn(const ValVector & inpvec,
		      const ValVector & tchvec,
		      const ValVector & nxtchvec) {
	    propFore(inpvec) ;
	    cntTrial++ ;
	    mother->cntTrial++ ;
	    propBack(tchvec,nxtchvec) ;
	    return learn() ;
	 } ;

	 Val runLearnWithMask(const ValVector & inpvec,
			      const ValVector & tchvec,
			      const ValVector & tchmask,
			      const ValVector & nxtchvec,
			      const ValVector & nxtchmask) {
	    propFore(inpvec) ;
	    cntTrial++ ;
	    mother->cntTrial++ ;
	    propBackWithMask(tchvec,tchmask,nxtchvec,nxtchmask) ;
	    return learn() ;
	 } ;

	 //--------------------------------------------------
	 // get sum of square error
      public:
	 Flt sumError() const { 
	    return output->sumError() + nxoutput->sumError(); 
	 } ;

   } ;
} ;


////////////////////////////////////////////////////////////////////////
#endif
