// -*- Mode:C++ -*-
//Header:
//File: ValVector.h
//Author: NODA, Itsuki
//Date: 1999/12/16
//

//ModifyHistory:
// 1999/12/16: Start to create this file
//EndModifyHistory:

#ifndef _Nnn_ValVector_h_
#define _Nnn_ValVector_h_
////////////////////////////////////////////////////////////////////////

#include "n3btype.h"
#include "n3utility.h"
#include "StrForm.h"

namespace Nnn {
   
   using namespace Itk;

   class ValVector : public WithDescriber, public vector<Val> {
	 //--------------------------------------------------
	 // labels for save/load/scan
      public:
	 static char* const LblTermHead = "V" ;
	 static char* const LblSize = "n" ;

	 //--------------------------------------------------
	 // constructor
      public:
	 typedef iterator Iterator ;
	 typedef const_iterator C_Iterator ;
	 typedef reverse_iterator RevIterator ;
	 typedef const_reverse_iterator C_RevIterator ;
      
	 //--------------------------------------------------
	 // constructor
      public:
	 ValVector() : vector<Val>() {} ;
	 ValVector(Size n, const Val & value = 0.0) : vector<Val>(n,value) {} ;
	 ValVector(const ValVector & src) : vector<Val>(src) {} ;

	 virtual ~ValVector() { } ;

	 //--------------------------------------------------
	 // access
      public:
	 Val & v(Index i) { return (*this)[i] ; } ;
	 const Val & v(Index i) const { return (*this)[i] ; } ;
      
	 //--------------------------------------------------
	 // copy vector
      public:
	 ValVector & copyFrom(const ValVector & src) ;
	 ValVector & copyTo(ValVector & dst) const {
	    return dst.copyFrom(*this) ;
	 } ;

	 //--------------------------------------------------
	 // duplicate
      public:
	 ValVector & duplicate(Bool copyvalp = True) const ;

	 //--------------------------------------------------
	 // set random
      public:
	 void randomize(const Val & minval, const Val & maxval) ;

	 //--------------------------------------------------
	 // set constant value
      public:
	 ValVector & set(const Val & val) ;
	 ValVector & setAll(const Val & val) { return set(val) ; } ;

	 ValVector & set(const Val & val, UInt from, UInt to) ;

	 //--------------------------------------------------
	 // increment by vector
      public:
	 ValVector & inc(const ValVector & v) ;

	 //--------------------------------------------------
	 // multiply 
      public:
	 ValVector & mul(const ValVector & v) ;

	 //--------------------------------------------------
	 // mask
      public:
	 ValVector & mask(const ValVector & mask) {
	    return mul(mask) ;
	 } ;

	 //--------------------------------------------------
	 // bmask
	 //  binary masking.  if mask[i] == 0.0, then v(i) = 0.0 ;
      public:
	 ValVector & bmask(const ValVector & mask) ;

	 //--------------------------------------------------
	 // statistics
      public:
	 Val min() const ;
	 Val max() const ;

	 Val sum() const ;
	 Val sqsum() const ;
	 Val ave() const ;
	 Val div() const ;
	 Val sdiv() const ;

	 void getMinMax(Val & min, Val & max) const ;
	 void getAveDiv(Val & ave, Val & div) const ;

	 //--------------------------------------------------
	 // save/load/scan
      public:
	 virtual void save(ostream& ostr) const ;
	 virtual void save(ostream& ostr, 
			   const SubString & indent) const ;
	 virtual void saveAsMatrix(ostream& ostr, 
				   UInt m,
				   const SubString & indent) const ;

	 virtual void load(istream & istr) {
	    StrForm * sform = StrForm::scan(istr) ;
	    scan(*sform) ;
	    delete sform ;
	 } ;
	 virtual void scan(StrForm & sform) ;

	 //--------------------------------------------------
	 // describe
      public:
	 static UInt formatWidth ;
	 static UInt formatPrecision ;
	 static void setFormat(Int width,Int prec) {
	    formatWidth = width ;
	    formatPrecision = prec ;
	 } ;
	 static void resetFormat() { 
	    setFormat(NNN_VAL_FORMAT_W,NNN_VAL_FORMAT_P) ; } ;

#define NNN_VALVEC_FORMAT NNN_VAL_FORM(formatWidth,formatPrecision)

      public:
	 virtual void describe(ostream & ostr, 
			       const Bool detailp = True) const {
	    //ostr << "#ValVector[" << size() << "|" ;
	    ostr << "#V[" << size() << "|" ;
	    for(C_Iterator i = begin() ; i < end() ; i++) {
	       if(i != begin()) ostr << "," ;
	       ostr << NNN_VALVEC_FORMAT << *i ;
	    } ;
	    ostr << "]" ;

	    if(detailp) {
	       ostr << endl ;
	    } 
	 } ;
   } ;


} ;
////////////////////////////////////////////////////////////////////////
#endif
