## -*- Mode: Ruby -*-

require 'gtk' ;
require "itkUtility" ;
require "GtkBufferedDrawingArea.rb" ;

module NnnViewer

  ##======================================================================
  ## class UnitWidget

  class Unit < Gtk::BufferedDrawingArea

    attr :valmax	,true ;
    attr :val		,true ;

    ##------------------------------------------------------------
    ## initialize

    def initialize(sizeX = 10, sizeY = 10)
      super(sizeX,sizeY) ;

      @valmax = 1.0 ;
      @val = 0.0 ;

      @mutex = Mutex.new() ;

    end

    ##------------------------------------------------------------
    ## setup

    def setup()
      super() ;

      @gcBlack = newColoredGCbyRGB(0x0000, 0x0000, 0x0000) ;
      @gcWhite = newColoredGCbyRGB(0xFFFF, 0xFFFF, 0xFFFF) ;
      @gcRed   = newColoredGCbyRGB(0xFFFF, 0x0000, 0x0000) ;
      @gcGreen = newColoredGCbyRGB(0x0000, 0xFFFF, 0x0000) ;
      @gcBlue  = newColoredGCbyRGB(0x0000, 0x0000, 0xFFFF) ;
      @gcGrey  = newColoredGCbyRGB(0x8000, 0x8000, 0x8000) ;
    end

    ##------------------------------------------------------------
    ## clear

    def clear(gc = @gcGrey)
      b = @buffer ;
      b.draw_rectangle(gc,TRUE,0,0,@xsize,@ysize) ;
      b.draw_rectangle(@gcBlack,FALSE,0,0,@xsize-1,@ysize-1) ;
    end

    ##------------------------------------------------------------
    ## redraw

    def redraw(exposep = FALSE)

      setup() if (@buffer.nil?) ;

      if(@val > 0) 
	r = @val/@valmax ;
	bggc = @gcWhite ;
	fggc = @gcRed ;
      else
	r = (1 + @val/@valmax) ;
	bggc = @gcBlue ;
	fggc = @gcWhite ;
      end

      x0 = (1 - r) * (@xsize / 2) + 1 ;
      dx = r * @xsize - 1;
      y0 = (1 - r) * (@ysize / 2) + 1 ;
      dy = r * @ysize - 1;
      
      clear(bggc) ;
      @buffer.draw_rectangle(fggc,TRUE,x0,y0,dx,dy) ;

      if(exposep) 
	draw(Gdk::Rectangle.new(0,0,@xsize,@ysize)) ;
      end
    end

    ##------------------------------------------------------------
    ## changeVal

    def changeVal(newval) 
      @val = newval ;
      redraw(TRUE) ;
    end

  end

end


