#include "Elman3SigLyr.h"

using namespace Itk ;
using namespace Nnn ;

//======================================================================

#include "TrainSetBase.h"

class Foo1TrainSet : public TrainSetBase {
   public:
      Val x ;
      static const Val dx = 0.1 ;
      static const Val sx = M_PI/4.0 ;

      static Val Psin(Val x) { return (1.0 + 0.5 * sin(x)) / 2.0 ; } ;
      static Val Pcos(Val x) { return (1.0 + 0.5 * cos(x)) / 2.0 ; } ;

      Foo1TrainSet() { x = 0 ; } ;

      virtual UInt sizeInp() const { return 2 ; } ;
      virtual UInt sizeTch() const { return 2 ; } ;

      virtual void get(ValVector & inp, ValVector & tch) {
	 x += dx ;
	 inp[0] = Psin(x + fltRand(-2 * dx, 2 * dx)) ;
	 inp[1] = 0 ;
	 tch[0] = Psin(x) ; tch[1] = Psin(x+sx) ;
      } ;
} ;

void test1sub(Elman3SigLyr & net) {

   Foo1TrainSet tset ;

   ValVector inp(tset.sizeInp()) ;
   ValVector tch(tset.sizeTch()) ;

   UInt m = 500000 ;

   Val err = 0.0 ;

   for(UInt i = 0 ; i < m ; i++) {
      tset.get(inp,tch) ;
      net.runLearn(inp,tch) ;
      err *= 0.9 ;
      err += net.sumError() ;
      if(i % 10000 == 0) cout << setw(7) << i << ":" << sqrt(err) << endl ;
   } ;
   
   ValVector out(tset.sizeTch()) ;
   for(UInt i = 0 ; i < 100 ; i++) {
      tset.get(inp,tch) ;
      net.run(inp) ;
      net.getNetOutput(out) ;
      if(i % 1 == 0) 
	 cout << inp << "/" << out << "/" << tch << endl ;
   }

}

void test1() { 
   ITK_MSG("====================== test 1 ======================") ;

   Elman3SigLyr net("Elman",2,6,2,1.0,0.01,0.0) ;
   net.randomize(-2,2) ;
   test1sub(net) ;
   net.save("foo2.net") ;
}

//--------------------------------------------------

void test2() { 
   ITK_MSG("====================== test 2 ======================") ;
   Elman3SigLyr net ;
   net.load("foo2.net") ;
   test1sub(net) ;
   net.save("foo2.net") ;
}

//======================================================================

class Foo3TrainSet : public TrainSetBase {
   public:
      Val x ; 
      Val y ;
      Val py ;

      virtual UInt sizeInp() const { return 2 ; } ;
      virtual UInt sizeTch() const { return 2 ; } ;

      virtual void get(ValVector & inp, ValVector & tch) {
	 x = (Flt)intRand(2) ;
	 y = (Flt)intRand(2) ;
	 inp[0] = x ; inp[1] = y ;
	 tch[0] = tch[1] ;
	 tch[1] = (x + y > 0.5 ) ? 1.0 : 0.0 ;
	 py = y ;
      } ;
} ;

void test3sub(Elman3SigLyr & net) {

   Foo3TrainSet tset ;

   ValVector inp(tset.sizeInp()) ;
   ValVector tch(tset.sizeTch()) ;

   UInt m = 500000 ;

   Val err = 0.0 ;

   for(UInt i = 0 ; i < m ; i++) {
      tset.get(inp,tch) ;
      net.runLearn(inp,tch) ;
      err *= 0.9 ;
      err += net.sumError() ;
      if(i % 10000 == 0) cout << setw(7) << i << ":" << sqrt(err) << endl ;
   } ;
   
   ValVector out(tset.sizeTch()) ;
   for(UInt i = 0 ; i < 10 ; i++) {
      tset.get(inp,tch) ;
      net.run(inp) ;
      net.getNetOutput(out) ;
      if(i % 1 == 0) 
	 cout << inp << "/" << out << "/" << tch << endl ;
   }

}

void test3() { 
   ITK_MSG("====================== test 3 ======================") ;

   Elman3SigLyr net("Elman",2,6,2,1.0,0.01,0.0) ;
   net.randomize(-2,2) ;
   test3sub(net) ;
   net.save("foo2.net") ;
}

//--------------------------------------------------

void test4() { 
   ITK_MSG("====================== test 4 ======================") ;
   Elman3SigLyr net ;
   net.load("foo2.net") ;
   test3sub(net) ;
   net.save("foo2.net") ;
}

//======================================================================
int main(int argc, char* argv[]) {
   //test1() ;
   test2() ;
   //test3() ;
   //test4() ;

   return 0;
}





