#include "Elman5L.h"
#include "TrainSetBase.h"

using namespace Itk ;
using namespace Nnn ;

//======================================================================
class FooTrainSet : public TrainSetBase {
   public:
      Val x ;
      static const Val dx = 0.1 ;
      static const Val sx = M_PI/4.0 ;

      FooTrainSet() { x = 0 ; } ;

      static Val q(Val v, Val step) {
	 return (Flt)((Int)(v/step)) ;
      } ;
	 

      virtual UInt sizeInp() const { return 1 ; } ;
      virtual UInt sizeTch() const { return 2 ; } ;

      virtual void get(ValVector & inp, ValVector & tch) {
	 x += dx ;
	 //inp[0] = sin(x + fltRand(-2 * dx, 2 * dx)) ;
	 inp[0] = sin(q(x,2 * dx)) ;
	 tch[0] = sin(x) ; 
	 tch[1] = sin(x+sx) ;
      } ;
} ;

FooTrainSet TSet ;

void test1sub(Elman5L & net) {

   ValVector inp(TSet.sizeInp()) ;
   ValVector tch(TSet.sizeTch()) ;

   UInt m = 500000 ;

   Val err = 0.0 ;

   for(UInt i = 0 ; i < m ; i++) {
      TSet.get(inp,tch) ;
      net.runLearn(inp,tch) ;
      err *= 0.9 ;
      err += net.sumError() ;
      if(i % 10000 == 0) cout << setw(7) << i << ":" << sqrt(err) << endl ;
   } ;
   
   ValVector out(TSet.sizeTch()) ;
   for(UInt i = 0 ; i < 100 ; i++) {
      TSet.get(inp,tch) ;
      net.run(inp) ;
      net.getNetOutput(out) ;
      if(i % 1 == 0) 
	 cout << inp << "/" << out << "/" << tch << endl ;
   }

}

void gen_save() { 
   ITK_MSG("====================== test 1 ======================") ;

   Elman5L net;
   net.alloc("Elman5L",TSet.sizeInp(),5,5,5,TSet.sizeTch(),1.0,0.01,0.0) ;
   net.randomize(-1,1) ;
   test1sub(net) ;
   net.save("foo3.net") ;
}

//--------------------------------------------------

void load_run_save() { 
   ITK_MSG("====================== test 2 ======================") ;
   Elman5L net ;
   net.load("foo3.net") ;
   test1sub(net) ;
   net.save("foo3.net") ;
}

//======================================================================
int main(int argc, char* argv[]) {
   load_run_save() ;

   return 0;
}





