#include "AuElman5L.h"
#include "TrainSetBase.h"

using namespace Itk ;
using namespace Nnn ;

//======================================================================
class FooTrainSet : public TrainSetBase {
   public:
      Val x ;
      Val y ; 
      static const Val dx = 0.345 ;
      static const Val dy = 0.123 ;

      FooTrainSet() { x = 0 ; y = 0 ; } ;

      virtual UInt sizeInp() const { return 1 ; } ;
      virtual UInt sizeTch() const { return 2 ; } ;

      virtual void get(ValVector & inp, ValVector & tch) {
	 x += dx ;
	 y += dy ;
	 //inp[0] = sin(x + fltRand(-2 * dx, 2 * dx)) ;
	 //inp[0] = sin(q(x,2 * dx)) ;
	 tch[0] = sin(x) ; 
	 tch[1] = sin(y) ;
      } ;
} ;

FooTrainSet TSet ;

//----------------------------------------------------------------------

void test1sub(AuElman5L & net) {

   ValVector inp(TSet.sizeInp()) ;
   ValVector tch(TSet.sizeTch()) ;

   UInt m = 500000 ;
   //UInt m = 1000;

   Val err = 0.0 ;

   for(UInt i = 0 ; i < m ; i++) {
      TSet.get(inp,tch) ;
      net.runLearn(tch) ;
      err *= 0.9 ;
      err += net.sumError() ;
      if(i % 10000 == 0) cout << setw(7) << i << ":" << sqrt(err) << endl ;
   } 

   cout << "-------------------- run(tch) ---------------------" << endl ;
   
   ValVector out(TSet.sizeTch()) ;
   for(UInt i = 0 ; i < 100 ; i++) {
      TSet.get(inp,tch) ;
      net.run(tch) ;
      net.getNetOutput(out) ;
      if(i % 1 == 0) 
	 cout << inp << "/" << out << "/" << tch << endl ;
   }

   cout << "-------------------- run() ---------------------" << endl ;

   for(UInt i = 0 ; i < 1000 ; i++) {
      TSet.get(inp,tch) ;
      net.run() ;
      net.getNetOutput(out) ;
      if(i % 1 == 0) 
	 cout << inp << "/" << out << "/" << tch << endl ;
   }
}

//----------------------------------------------------------------------

void test2() {
   AuElman5L net ;
   net.load("foo5.net") ;

   ValVector inp(TSet.sizeInp()) ;
   ValVector tch(TSet.sizeTch()) ;
   ValVector err(5) ;
   ValVector out(TSet.sizeTch()) ;
   ValVector ctx(5) ;
   

   UInt m = 10 ;

   for(UInt i = 0 ; i < m ; i++) {
      TSet.get(inp,tch) ;
      net.runLearn(tch) ;
   }

   TSet.get(inp,tch) ;
   net.propFore(True) ;
   net.output->getOut(out) ;
   cout << "out=" << out << endl ;
   cout << "tch=" << tch << endl ;

   net.propBack(tch) ;
   net.context->getOut(ctx) ;
   net.context->getErr(err) ;
   cout << "ctx=" << ctx << endl ;
   cout << "err=" << err << endl ;

   net.adjustContext(1.0) ;
   net.context->getOut(ctx) ;
   cout << "ctx=" << ctx << endl ;

   net.propFore(False) ;
   net.output->getOut(out) ;
   cout << "out=" << out << endl ;
}
   


//======================================================================

void gen_save() { 
   ITK_MSG("====================== test 1 ======================") ;

   AuElman5L net;
   net.alloc("AuElman5L",5,5,5,TSet.sizeTch(),1.0,0.01,0.0) ;
   net.randomize(-1,1) ;
   test1sub(net) ;
   net.save("foo5.net") ;
}

//--------------------------------------------------

void load_run_save() { 
   ITK_MSG("====================== test 2 ======================") ;
   AuElman5L net ;
   net.load("foo5.net") ;
   test1sub(net) ;
   net.save("foo5.net") ;
}

//======================================================================
int main(int argc, char* argv[]) {
   load_run_save() ;
   //test2() ;

   return 0;
}





