#include "VElman5L.h"
#include "TrainSetBase.h"

using namespace Itk ;
using namespace Nnn ;

#define NETFILE "foo8.net"

//======================================================================
class TsSin : public TrainSetBase {
   public:
      Val x ;
      static const Val dx = 0.1 ;
      //static const Val sx = M_PI/4.0 ;
      static const Val sx = M_PI/8.0 ;

      TsSin() { x = 0 ; } ;

      static Val q(Val v, Val step) {
	 return (Flt)((Int)(v/step)) ;
      } ;
	 

      virtual UInt sizeInp() const { return 1 ; } ;
      virtual UInt sizeTch() const { return 2 ; } ;

      virtual void get(ValVector & inp, ValVector & tch) {
	 x += dx ;
	 //inp[0] = sin(x + fltRand(-2 * dx, 2 * dx)) ;
	 //inp[0] = sin(q(x,2 * dx)) ;
	 inp[0] = sin(x) ;
	 tch[0] = sin(x) ; 
	 tch[1] = sin(x+sx) ;
      } ;
} ;

//======================================================================
class TsDelay : public TrainSetBase {
   public:
      Val x ;
      Val px ;
      Val ppx ;

      TsDelay() { x = 0 ; px = 0 ; ppx = 0 ;} ;
      
      virtual UInt sizeInp() const { return 1 ; } ;
      virtual UInt sizeTch() const { return 3 ; } ;

      virtual void get(ValVector & inp, ValVector & tch) {
	 ppx = px ;
	 px = x ;
	 x = fltRand(-50,50) ;
	 inp[0] = x ;
	 tch[0] = x ;
	 tch[1] = px ;
	 tch[2] = ppx ;
      } ;
} ;
      

      

//======================================================================

//TsSin TSet ;
TsDelay TSet ;

void test1sub(VElman5L & net) {

   ValVector inp(TSet.sizeInp()) ;
   ValVector tch(TSet.sizeTch()) ;

   //UInt m = 500000 ;
   UInt m = 100000 ;

   Val err = 0.0 ;

   for(UInt i = 1 ; i <= m ; i++) {
      TSet.get(inp,tch) ;
      net.runLearn(inp,tch) ;
      const Val r = 0.9 ;
      err *= r ;
      err += net.sumError() ;
      if((i) % 1000 == 0) {
	 cout << setw(7) << i<< ":" << sqrt(err)
	      << endl ;
	 net.save(NETFILE) ;
      }
   } 
   
   ValVector out(TSet.sizeTch()) ;
   m = 100;
   for(UInt i = 0 ; i < m ; i++) {
      TSet.get(inp,tch) ;
      net.run(inp) ;
      net.getNetOutput(out) ;
      if(i % 1 == 0) 
	 cout << inp << "/" << out << "/" << tch << endl ;
   }

}

//--------------------------------------------------

void load_run_save() { 
   ITK_MSG("====================== test 2 ======================") ;
   VElman5L net ;
   net.load(NETFILE) ;
   test1sub(net) ;
}

//======================================================================
int main(int argc, char* argv[]) {
   load_run_save() ;

   return 0;
}





