## -*- Mode: Ruby -*-

require "NnnViewXYPlane2.rb" ;

module NnnViewer

  ##======================================================================
  ## class UnitWidget

  class SoccerField < XYPlane2

    class Player
      attr :state	,true ;
      attr :side	,true ;
      attr :unum	,true ;
      attr :x		,true ;
      attr :y		,true ;
      attr :bdirx	,true ;
      attr :bdiry	,true ;
      attr :hdirx	,true ;
      attr :hdiry	,true ;
      attr :gc		,true ;
      attr :fillp	,true ;

      def initialize(side,unum = 0)
	@side = side ;
	@unum = unum ;
	@x = @y = 0 ;
	@bdirx = @bdiry = 0 ;
	@hdirx = @hdiry = 0 ;
	@gc = nil ;
      end

      def bdirRad()
	return atan2(bdiry,bdirx) ;
      end
      
      def hdirRad()
	return atan2(hdiry,hdirx) ;
      end

      def isLeft()
	return @side == 'left' ;
      end

      def isRight()
	return @side == 'right' ;
      end

      def setPos(x,y)
	@x = x ; @y = y ;
      end

      def setBDirXY(x,y) 
	@bdirx = x ; @bdiry = y ;
      end

      def setHDirXY(x,y) 
	@bdirx = x ; @bdiry = y ;
      end

    end

    attr :player	,true ;

    ##------------------------------------------------------------
    ## initialize

    def initialize(parent)
      @mag = 6 ;
      @margin = 20 ;
      @wmargin = @margin * 2 ;
      @width = 68.0 ;		@widthH = @width/2 ;
      @length = 106.0 ;		@lengthH = @length/2 ;
      @penaltyWidth = 40.32 ;	@penaltyWidthH = @penaltyWidth/2 ;
      @penaltyLength = 16.5 ;	@penaltyLengthH = @penaltyLength/2 ;
      @goalAreaWidth = 18.32 ;	@goalAreaWidthH = @goalAreaWidth/2 ;
      @goalAreaLength = 5.5 ;	@goalAreaLengthH = @goalAreaLength/2 ;
      @goalWidth = 14.02 ;	@goalWidthH = @goalWidth/2 ;
      @goalDepth = 2 ;


      super(parent, 
	    @mag * @length + @wmargin, @mag * @width + @wmargin, 
	    "field") ;
      @scaleX = @scaleY = @mag ;

      @player = [] ;
      @ballPos = [0,0] ;
    end

    ##------------------------------------------------------------
    ## draw Background
    
    def ar(r)
      return r * @mag ;
    end

    def drawBackground()
      @fieldGC = @image.newColoredGC("DarkGreen") ;
      @whiteGC = @image.whiteGC ;
      @blackGC = @image.blackGC ;
      
      @leftGC = @image.newColoredGC("gold") ;
      @rightGC = @image.newColoredGC("red") ;
      @otherGC = @image.newColoredGC("blue") ;

      @image.clear(@fieldGC) ;

      ## around the field
      @image.draw_lines(@whiteGC,
			[ [ax(-@lengthH),ay(-@widthH)],
			  [ax(-@lengthH),ay( @widthH)],
			  [ax( @lengthH),ay( @widthH)],
			  [ax( @lengthH),ay(-@widthH)],
			  [ax(-@lengthH),ay(-@widthH)]]) ;
      ## center line
      @image.draw_line(@whiteGC,
		       ax(        0),ay(-@widthH),ax(        0),ay( @widthH)) ;

      ## center circle
      @image.draw_circle_centered(@whiteGC,FALSE,ax(0),ay(0),ar(2 * 9.15)) ;

      ## left Goal
      @image.draw_rectangle(@blackGC,TRUE,
			    ax(-@lengthH-@goalDepth),ay(-@goalWidthH),
			    ar(@goalDepth),ar(@goalWidth)) ;
      ## right Goal
      @image.draw_rectangle(@blackGC,TRUE,
			    ax( @lengthH),ay(-@goalWidthH),
			    ar(@goalDepth),ar(@goalWidth)) ;

      ## left penalty area
      @image.draw_lines(@whiteGC,
			[ [ax(-@lengthH),               ay(-@penaltyWidthH)],
			  [ax(-@lengthH+@penaltyLength),ay(-@penaltyWidthH)],
			  [ax(-@lengthH+@penaltyLength),ay( @penaltyWidthH)],
			  [ax(-@lengthH),               ay( @penaltyWidthH)]]);

      ## right penalty area
      @image.draw_lines(@whiteGC,
			[ [ax( @lengthH),               ay( @penaltyWidthH)],
			  [ax( @lengthH-@penaltyLength),ay( @penaltyWidthH)],
			  [ax( @lengthH-@penaltyLength),ay(-@penaltyWidthH)],
			  [ax( @lengthH),               ay(-@penaltyWidthH)]]);


      ## left goal area
      @image.draw_lines(@whiteGC,
			[ [ax(-@lengthH),                ay(-@goalAreaWidthH)],
			  [ax(-@lengthH+@goalAreaLength),ay(-@goalAreaWidthH)],
			  [ax(-@lengthH+@goalAreaLength),ay( @goalAreaWidthH)],
			  [ax(-@lengthH),                ay( @goalAreaWidthH)]
			  ]);
      ## right goal area
      @image.draw_lines(@whiteGC,
			[ [ax( @lengthH),                ay( @goalAreaWidthH)],
			  [ax( @lengthH-@goalAreaLength),ay( @goalAreaWidthH)],
			  [ax( @lengthH-@goalAreaLength),ay(-@goalAreaWidthH)],
			  [ax( @lengthH),                ay(-@goalAreaWidthH)]
			  ]);

    end

    ##------------------------------------------------------------
    ## draw player (low level)

    DEG2RAD = PI / 180.0 ;
    RAD2DEG = 180.0 / PI ;

    def _drawPlayerDeg(gc,realp,x,y,bdir,hdir=0,unum=0) 
      _drawPlayerRad(gc,realp,x,y,bdir * DEG2RAD, hdir * DEG2RAD, unum) ;
    end

    def _drawPlayerRad(gc,realp,x,y,bdir,hdir=0,unum=0) 
      br = 15 ;
      bdirlen = br + 10 ;
      
      @image.draw_circle_centered(gc,realp,ax(x),ay(y),br) ;
      @image.draw_line(gc,ax(x),ay(y),
		       ax(x)+(bdirlen * cos(bdir)),
		       ay(y)+(bdirlen * sin(bdir))) ;
      @image.draw_line(@whiteGC,ax(x),ay(y),
		       ax(x)+(br * cos(hdir)),
		       ay(y)+(br * sin(hdir))) ;
    end

    ##------------------------------------------------------------
    ## draw player

    def _drawBall(realp,x,y) 
      br = 10 ;
      @image.draw_circle_centered(@whiteGC,realp,ax(x),ay(y),br) ;
    end

    ##------------------------------------------------------------
    ## register player

    def _registerPlayer(side,unum)
      player = Player.new(side,unum);
      @player.push(player) ;
      return player ;
    end

    def getPlayer(side,unum) 
      @player.each do |p|
	if(p.side == side && p.unum == unum)
	  return p ;
	end
      end
      return nil ;
    end

    def registerPlayer(side,unum)
      p = getPlayer(side,unum) ;
      if(p == nil)
	p = _registerPlayer(side,unum) ;
      end
      return p ;
    end

    def registerColoredPlayer(side,unum,color,fillp = TRUE)
      pl = registerPlayer(side,unum) ;
      pl.gc = @image.newColoredGC(color) ;
      pl.fillp = fillp ;
      return pl ;
    end

    ##------------------------------------------------------------
    ## draw all players and ball

    def drawAllPlayer()
      @player.each do |pl|
	if(pl.gc == nil) 
	  if   (pl.isLeft())
	    _drawPlayerRad(@leftGC,TRUE,
			   pl.x,pl.y,pl.bdirRad(),pl.hdirRad(),pl.unum);
	  elsif(pl.isRight())
	    _drawPlayerRad(@rightGC,TRUE,
			   pl.x,pl.y,pl.bdirRad(),pl.hdirRad(),pl.unum);
	  else
	    _drawPlayerRad(@otherGC,TRUE,
			   pl.x,pl.y,pl.bdirRad(),pl.hdirRad(),pl.unum);
	  end
	else
	  _drawPlayerRad(pl.gc,pl.fillp,
			 pl.x,pl.y,pl.bdirRad(),pl.hdirRad(),pl.unum);
	end
      end
    end

    def drawBall()
      _drawBall(TRUE,@ballPos[0],@ballPos[1]) ;
    end

    def drawAllPlayerBall()
      if(@leftGC == nil || @rightGC == nil || @otherGC == nil)
	return ;
      end

      drawAllPlayer() ;
      drawBall() ;
    end

    ##------------------------------------------------------------
    ## flush

    def drawFlush()
      setup() if(!@isSetuped) ;
      
##      restoreBuffer() ;
      
      drawAllPlayerBall() ;
      
      @image.exposeAll() ;

    end
  end
end



