// -*- Mode:C++ -*-
//Header:
//File: FullStateInfo.h
//Title: Full State Information (for version 7)
//Author: NODA, Itsuki
//Date: 2001/03/11
//

//ModifyHistory:
// 2001/03/11: Start to create this file
//EndModifyHistory:

#ifndef _saloo_FullStateInfo_h_
#define _saloo_FullStateInfo_h_
////////////////////////////////////////////////////////////////////////

#include "itk/itk.h"
#include "sctypes.h"
#include "CycleStep.h"

namespace Saloo {

   using namespace Itk ;
   using namespace Soccer ;

   //============================================================
   // full state

   class FullStateInfo : public WithDescriber {

	 //==================================================
	 // class Player

	 class Player : public WithDescriber {
	       //----------------------------------------
	       // member
	    public:
	       ScWord * name ;
	       Side side ;
	       UInt unum ;
	       PosDir posdir ;
	       Vec vel ;
	       Flt stamina ;
	       Flt effort ;
	       Flt recovery ;

	       //----------------------------------------
	       // scan
	    public:
	       Bool scan(SimpleSexp & s) ;
	       Bool scan(SimpleSexp * s) { return scan(*s) ; } ;
	       Bool scan(Buffer & buffer) { 
		  return scan(SimpleSexp::scan(buffer)) ;
	       } ;
	       Bool scan(SubString & str) {
		  Buffer b(str) ;
		  return scan(b) ;
	       } ;

	       //----------------------------------------
	       // describe
	    public:
	       virtual void describe(ostream & ostr, 
				     const Bool detailp = True) const {
		  ostr << "#Player[" << name->name << "]"  ;

		  if(detailp) {
		     ostr << endl ;
		     ostr << "\t" << "posdir=" << posdir << endl ;
		     ostr << "\t" << "vel=" << vel << endl ;
		     ostr << "\t" << "stamina=" << stamina 
			  << "/" << effort << "/" << recovery
			  << endl ;
		  }
	       } ;
	       
	 } ;

	 //==================================================
	 // Main Body (HearInfo)

	 //--------------------------------------------------
	 // member
      public:
	 CycleStep time ;
	 ScWord * playmode ;
	 ScWord * viewQuality ;
	 ScWord * viewWidth ;
	 UInt scoreL ;
	 UInt scoreR ;
	 Pos ballpos ;
	 Vec ballvel ;
	 UInt playerN ;
	 Player player[SC_N_PLAYER] ;

	 //--------------------------------------------------
	 // scan
      public:
	 Bool scan(SimpleSexp & s) ;
	 Bool scan(SimpleSexp * s) { return scan(*s) ; } ;
	 Bool scan(Buffer & buffer) { 
	    return scan(SimpleSexp::scan(buffer)) ;
	 } ;
	 Bool scan(SubString & str) {
	    Buffer b(str) ;
	    return scan(b) ;
	 } ;

	 //--------------------------------------------------
	 // describe
      public:
	 virtual void describe(ostream & ostr, 
			       const Bool detailp = True) const {
	    ostr << "#FullStateInfo[" << time << "]" ;
	    
	    if(detailp) {
	       ostr << endl ;
	       ostr << "\t" << "playmode=" << *playmode << endl ;
	       ostr << "\t" << "view=" << *viewQuality 
		    << "/" << *viewWidth << endl ;
	       ostr << "\t" << "score=" << scoreL << ":" << scoreR << endl ;
	       ostr << "\t" << "ball=" << ballpos << "/" << ballvel << endl ;
	       ostr << "\t" << "[player] N = " << playerN << endl ;
	       for(UInt i = 0 ; i < playerN ; i++) {
		  ostr << "\t\t" << "player[" << i << "]=" ;
		  player[i].describe(ostr,detailp) ;
	       }
	    } 
	    
	 } ;
   } ;
} ;

////////////////////////////////////////////////////////////////////////
#endif
