// -*- Mode:C++ -*-
//Header:
//File: SenseBodyInfo.h
//Title: SenseBody
//Author: NODA, Itsuki
//Date: 2001/03/05
//

//ModifyHistory:
// 2001/03/05: Start to create this file
//EndModifyHistory:

#ifndef _saloo_SenseBodyInfo_h_
#define _saloo_SenseBodyInfo_h_
////////////////////////////////////////////////////////////////////////

#include "itk/itk.h"
#include "sctypes.h"
#include "CycleStep.h"

namespace Saloo {

   using namespace Itk ;
   using namespace Soccer ;

   //--------------------------------------------------
   // sense body

   class SenseBodyInfo : public WithDescriber {
	 
	 //==================================================
	 // Main Body (HearInfo)

	 //--------------------------------------------------
	 // member
      public:
	 CycleStep time ;
	 
	 ViewQuality viewQuality ;
	 ViewWidth viewWidth ;

	 Flt stamina ;
	 Flt effort ;
	 Flt speed ;
	 Flt headAngle ;

	 UInt countKick ;
	 UInt countDash ;
	 UInt countTurn ;
	 UInt countSay ;
	 UInt countCatch ;
	 UInt countTurnNeck ;
	 UInt countChangeView ;
	 UInt countSenseBody ;
	 UInt countMove ;

	 //--------------------------------------------------
	 // reset

	 void reset() {
	    time = 0 ; 
	    viewQuality = VQ_high ; viewWidth = VW_normal ;
	    stamina = 0.0 ; effort = 1.0 ; 
	    speed = 0.0 ; headAngle = 0.0 ;
	    countKick = 0 ; countDash = 0 ; 
	    countTurn = 0 ; countCatch = 0 ; 
	    countTurnNeck = 0 ; countSay = 0 ;
	    countChangeView = 0 ;countSenseBody = 0 ;
	 } ;

	 //--------------------------------------------------
	 // set

	 void set(const SenseBodyInfo& src) {
	    time = src.time ; 
	    viewQuality = src.viewQuality ; viewWidth = src.viewWidth ;
	    stamina = src.stamina ; effort = src.effort ; 
	    speed = src.speed ; headAngle = src.headAngle ;
	    countKick = src.countKick ; countDash = src.countDash ; 
	    countTurn = src.countTurn ; countCatch = src.countCatch ; 
	    countTurnNeck = src.countTurnNeck ; countSay = src.countSay ;
	    countChangeView = src.countChangeView ; 
	    countSenseBody = src.countSenseBody ;
	 } ;

	 //--------------------------------------------------
	 // scan

	 Bool scan(SimpleSexp & s) ;
	 Bool scan(SimpleSexp * s) { return scan(*s) ; } ;
	 Bool scan(Buffer & buffer) { 
	    return scan(SimpleSexp::scan(buffer)) ;
	 } ;
	 Bool scan(SubString & str) {
	    Buffer b(str) ;
	    return scan(b) ;
	 } ;

	 //--------------------------------------------------
	 // describe
      public:
	 virtual void describe(ostream & ostr, 
			       const Bool detailp = True) const {

	    ostr << "#SenseBodyInfo[t=" << time 
		 << ":view=(" ;
	    switch(viewQuality) {
	       case VQ_high: ostr << "high" ; break ;
	       case VQ_low:  ostr << "low" ; break ;
	    }
	    ostr << "," ;
	    switch(viewWidth) {
	       case VW_wide: ostr << "wide" ; break ;
	       case VW_normal: ostr << "normal" ; break ;
	       case VW_narrow: ostr << "narrow" ; break ;
	    }

	    ostr << "), stamina=" << stamina ;
	    ostr << ", effort=" << effort ;
	    ostr << ", speed=" << speed ;
	    ostr << ", head=" << headAngle ;
	    ostr << ", kick=" << countKick ;
	    ostr << ", dash=" << countDash ;
	    ostr << ", turn=" << countTurn ;
	    ostr << ", say=" << countSay ;
	    ostr << ", catch=" << countCatch ;
	    ostr << ", turn_neck=" << countTurnNeck ;
	    ostr << ", change_view=" << countChangeView ;
	    ostr << ", sense_body=" << countSenseBody ;
	    ostr << "]" ;
	    if(detailp) ostr << endl ;
	 } ;
   } ;
} ;

////////////////////////////////////////////////////////////////////////
#endif
