// -*- Mode:C++ -*-
//Header:
//File: BasePlayer.cc
//Title: Base Player for Soccer Server 7
//Author: NODA, Itsuki
//Date: 2001/03/03
//

//ModifyHistory:
// 2001/03/03: Start to create this file
//EndModifyHistory:

#include "ThreadedPlayer.h"

namespace Saloo {


   //======================================================================
   // action thread

   //--------------------------------------------------
   // send command pre op

   Bool ThreadedPlayer::sendCommandPre(Bool bodyp) {
      cvSend.lock() ; 
      {
	 if(bodyp) {
	    while(nextSendBodyTime.isFuture()) {
	       cvSend.waitUntil(nextSendBodyTime) ;
	    }
	 }
	 
	 while(nextSendTime.isFuture()) {
	    cvSend.waitUntil(nextSendTime) ;
	 }

	 // update timestamp ??? right? 
	 
	 prevSendTime.update() ;
	 nextSendTime = prevSendTime + TimeVal(0,SendTic,0) ;

	 if(bodyp) {
	    prevSendBodyTime.update() ;
	    nextSendBodyTime = prevSendBodyTime + TimeVal(0,SendBodyTic,0) ;
	 }
      }
      // cvSend.unlock() ; /* unlock is located in sendCommandPost */
      
      return True ;
   } ;

   //--------------------------------------------------
   // send command post op

   Bool ThreadedPlayer::sendCommandPost(Bool bodyp) {
      cvSend.broadcast() ;
      cvSend.unlock() ;

      return True ;
   } ;
   

   //======================================================================
   // sensor thread

   //--------------------------------------------------
   // post operation for scanSensor

   Bool ThreadedPlayer::scanSensorPre(SimpleSexp & s) {
      cvSensor.lock() ;
      return True ;
   }

   //--------------------------------------------------
   // post operation for scanSensor

   Bool ThreadedPlayer::scanSensorPost(SimpleSexp & s, Bool result) {
      if(result) {
	 switch(sinfo.type) {
	    case SensorInfo::T_See : {  
	       result = analyzeSensorSee() ;
	       cvSee.lockSignal() ; 
	       break ; 
	    } 
	    case SensorInfo::T_Hear : { 
	       result = analyzeSensorHear() ;
	       cvHear.lockSignal() ; 
	       break ; 
	    }
	    case SensorInfo::T_SenseBody : { 
	       result = analyzeSensorSenseBody() ;
	       cvSenseBody.lockSignal() ; 
	       break ; 
	    }
	    case SensorInfo::T_FullState : { 
	       result = analyzeSensorFullState() ;
	       cvFullState.lockSignal() ; 
	       break ; 
	    }
	    default : {
	    }
	 }
	 cvSensor.signal() ;
      } 

      cvSensor.unlock() ;
      return result ;
   } ;

   //--------------------------------------------------
   // analizeSensor See/Hear/SenseBody/FullState

   Bool ThreadedPlayer::analyzeSensorSee() { return True ; } ;
   Bool ThreadedPlayer::analyzeSensorHear() { return True ; } ;
   Bool ThreadedPlayer::analyzeSensorSenseBody() { return True ; } ;
   Bool ThreadedPlayer::analyzeSensorFullState() { return True ; } ;

   //--------------------------------------------------
   // fork sensor thread

   Thread * ThreadedPlayer::forkSensor() {
      thrdSensor = new Thread(ThreadedPlayer::runSensor,this) ;
      return thrdSensor ;
   }
	    
   //--------------------------------------------------
   // outside function for sensor thread

   void * ThreadedPlayer::runSensor(void * p) {
      ThreadedPlayer * player = static_cast<ThreadedPlayer*>(p) ;
      return player->sensorMainLoop() ;
   } 

   //--------------------------------------------------
   // inside function for sensor thread

   void * ThreadedPlayer::sensorMainLoop() {
      while(True) {
	 cycleSensor() ;
      }
      return this ;
   }

} ;
