// -*- Mode:C++ -*-
//Header:
//File: sctypes.h
//Title: soccer basic types 
//Author: NODA, Itsuki
//Date: 2001/03/04
//EndHeader:

//ModifyHistory:
// 2001/03/04: Start to create this file
//EndModifyHistory:

#ifndef _sctypes_h_
#define _sctypes_h_
////////////////////////////////////////////////////////////////////////

#include "itk/itk.h"
#include "itk/Geo2D.h"

namespace Soccer {
   using namespace Itk ;

   //======================================================================
   // Server Info

   static const char DfltSSHost[] = "localhost" ;
   static const UInt DfltSSPort = 6000 ;
   static const UInt DfltSSOffCPort = 6001 ;
   static const UInt DfltSSOnCPort = 6002 ;	/* online coach */
   static const Flt DfltSSProto = 7.0 ;

   static const char DfltTeamName[] = "Saloo" ;
   static const char DfltTeamName2[] = "Monkey" ;

   //======================================================================
   // Buffer Size

   static const UInt DfltNameBufSize = 256 ;
   static const UInt DfltBufSize = 8192 ;

   //======================================================================
   // Side

   enum Side { Left, NoSide, Right } ;

   inline Bool isLeft(const Side side){ return (side == Left);};
   inline Bool isRight(const Side side){ return (side == Right);};

   inline Side otherside(Side side) {
      return (side == Left) ? Right 
	 :   (side == Right) ? Left 
	 :                     NoSide ;
   } ;

   inline Int sideSign(const Side side) {
      if(isLeft(side)) return 1 ;
      if(isRight(side)) return -1 ;
      return 0 ;
   } ;

   inline char sideChar(const Side side) {
      if      (isLeft(side))  return 'l' ;
      else if (isRight(side)) return 'r' ;
      else                    return '?' ;
   } ;

   inline const char * const sideString(Side side, Bool longp = True) {
      switch(side) {
	 case Left : return (longp ? "left"   : "l") ;
	 case Right: return (longp ? "right"  : "r") ;
	 default:    return (longp ? "noside" : "?") ;
      }
   }

   inline Side scanAsSide(const char c) {
      if(c == 'l') return Left ;
      if(c == 'r') return Right ;
      return NoSide ;
   } ;

   Side scanAsSide(const SimpleSexp & s) ;
   inline Side scanAsSide(const SimpleSexp * s) { return scanAsSide(*s) ;} ;

//======================================================================
// Field Parameters
//

//--------------------------------------------------
// Number of Objects

#define SC_N_PLAYER_PER_TEAM    11
#define SC_N_PLAYER     (SC_N_PLAYER_PER_TEAM * 2) 
#define SC_N_LINE       4 
#define SC_N_GOAL       2 
#define SC_N_FLAG       (6 + 6 + 4 + 22 + 14 + 1) 
/* corner, penalty, goal post, touchline, goalline, center mark */

#define SC_SEENOBJMAX   (1 + SC_N_PLAYER + SC_N_LINE + SC_N_GOAL + SC_N_FLAG)

//--------------------------------------------------
// Field Param

#define SC_PITCH_LENGTH         105.0 
#define SC_PITCH_WIDTH          68.0 
#define SC_PENALTY_AREA_LENGTH  16.5
#define SC_PENALTY_AREA_WIDTH   40.32 
#define SC_GOAL_WIDTH           14.02

//--------------------------------------------------
// Player Param

#define SC_PLAYER_SIZE          0.3
#define SC_PLAYER_DECAY         0.4
#define SC_PLAYER_RAND          0.1
#define SC_PLAYER_WEIGHT        60.0
#define SC_PLAYER_SPEED_MAX     1.0

#define SC_STAMINA_MAX          3500.0
#define SC_STAMINA_INC_MAX      20.0

#define SC_RECOVER_DEC_THR      0.3
#define SC_RECOVER_DEC          0.002
#define SC_RECOVER_MIN          0.5

#define SC_EFFORT_DEC_THR       0.3
#define SC_EFFORT_DEC           0.005
#define SC_EFFORT_INC_THR       0.6
#define SC_EFFORT_INC           0.01
#define SC_EFFORT_MIN           0.6

#define SC_HEAR_MAX             2
#define SC_HEAR_INC             1
#define SC_HEAR_DECAY           2

#define SC_INERTIA_MOMENT       5.0

#define SC_SENSE_BODY_STEP      100

// 1.04o (renamed cacheable -> catchable
#define SC_CATCHABLE_AREA_L     2.0
#define SC_CATCHABLE_AREA_W     1.0
#define SC_CATCH_PROBABILITY    1.0
#define SC_CATCH_BAN_CYCLE      5

#define SC_BALL_SIZE            0.085
#define SC_BALL_DECAY           0.94
#define SC_BALL_RAND            0.05
#define SC_BALL_WEIGHT          0.2
#define SC_BALL_SPEED_MAX       2.7

#define SC_WIND_FORCE           0.0
#define SC_WIND_DIR             0.0
#define SC_WIND_RAND            0.0

#define SC_KICKABLE_MARGIN      0.7
#define SC_KICKABLE_AREA (SC_BALL_SIZE + SC_PLAYER_SIZE + SC_KICKABLE_MARGIN)
#define SC_CKICK_MARGIN         1.0

#define SC_DASH_POWER_RATE      0.006
#define SC_KICK_POWER_RATE      0.016

#define SC_MAXPOWER             100.0
#define SC_MINPOWER             (-100.0)

#define SC_MAXMOMENT            180.0
#define SC_MINMOMENT            (-180.0)

//--------------------------------------------------
// Simulation Param

#define SC_DEFAULT_BODY_COMMAND_TIC_MSEC    100
#define SC_ESTIMATE_TIC_ALPHA           0.7
#define SC_ESTIMATE_TIC_CUTOFF          1.5

   const UInt SendTic = 1 ;  /* msec */
   const UInt SendBodyTic = SC_DEFAULT_BODY_COMMAND_TIC_MSEC ;

#define SC_DEFAULT_DASH_POW_POS_EFFECT  0.02
#define SC_DASH_POWER_RATE 0.006
#define SC_KICK_POWER_RATE 0.016

   //======================================================================
   // Referee Messages

#include "vocabulary.h"   
   extern ScWord ScWordTable[] ;

   ScWord * scanAsScWord(const SubString & s) ;

   ScWord * scanAsScWord(const SimpleSexp & s) ;
   inline ScWord * scanAsScWord(const SimpleSexp * s) {
      return scanAsScWord(*s) ;
   } ;

   ScWord * getScWord(const ScWordID & id) ;

   //--------------------------------------------------
   // view mode

   enum ViewWidth {
      VW_narrow = SC_WID_Narrow,
      VW_normal = SC_WID_Normal,
      VW_wide = SC_WID_Wide
   } ;

#define SC_VW_NAME(wmode) ScWordTable[wmode].name

   enum ViewQuality {
      VQ_high = SC_WID_High,
      VQ_low = SC_WID_Low
   } ;

#define SC_VQ_NAME(qmode) ScWordTable[qmode].name


   //======================================================================
   // Flag Table

   typedef Geo2D::Vec Pos ;

#include "landmarktable.h"   

   extern Landmark LandmarkTable[] ;

   Landmark * scanAsLandmark(const SimpleSexp & s) ;
   inline Landmark * scanAsLandmark(const SimpleSexp * s) {
      return scanAsLandmark(*s) ;
   } ;

   Landmark * scanAsLandmark(const SubString & s) ;

   Landmark * getLandmark(LandmarkID id) ;

   //--------------------------------------------------
   // Line Info

   enum LineID {
      LID_l = LMID_LineL,
      LID_t = LMID_LineT,
      LID_b = LMID_LineB,
      LID_r = LMID_LineR,
   } ;

   //======================================================================
   // XY Position + Angle

   typedef Geo2D::Vec Vec ;

   class PosDir {
      public:
	 Vec pos ;
	 Angle bdir ;    // body direction
	 Angle hdir ;    // head direction

	 //--------------------
	 // describe

	 void describe(ostream& ostr, 
		       Bool detailp = True) const {
	    ostr << "#PosDir[" << pos << "," 
		 << hdir << "/"<< bdir << "]" ;
	    if(detailp) ostr << endl ;
	 } ;
	 friend ostream& operator<< (ostream& ostr, 
				     const PosDir& pd) {
	    pd.describe(ostr,False) ;
	    return ostr ;
	 } ;
   } ;

} ;
////////////////////////////////////////////////////////////////////////
#endif
